@@ -60,9 +60,9 @@ Z* -------------------------------------------------------------------
 #define OrthoSaveLines 0xFF
 #define OrthoHistoryLines 0xFF
 
-#define cOrthoCharWidth 8
-#define cOrthoLeftMargin 3
-#define cOrthoBottomMargin 5
+#define cOrthoCharWidth DIP2PIXEL(8)
+#define cOrthoLeftMargin DIP2PIXEL(3)
+#define cOrthoBottomMargin DIP2PIXEL(5)
 
 #define CMD_QUEUE_MASK 0x3
 
@@ -497,7 +497,7 @@ void OrthoClear(PyMOLGlobals * G)
 void OrthoFeedbackIn(PyMOLGlobals * G, const char *buffer)
 {
   COrtho *I = G->Ortho;
-  if(G->HaveGUI) {
+  if(G->Option->pmgui) {
     if(I->feedback)
       QueueStrIn(I->feedback, buffer);
   }
@@ -508,10 +508,16 @@ void OrthoFeedbackIn(PyMOLGlobals * G, const char *buffer)
 int OrthoFeedbackOut(PyMOLGlobals * G, char *buffer)
 {
   COrtho *I = G->Ortho;
-  if(I->feedback)
-    return (QueueStrOut(I->feedback, buffer));
-  else
-    return (0);
+  if(!I->feedback)
+    return 0;
+
+  int status = QueueStrOut(I->feedback, buffer);
+
+  if (status && !SettingGetGlobal_b(G, cSetting_colored_feedback)) {
+    UtilStripANSIEscapes(buffer);
+  }
+
+  return status;
 }
 
 
@@ -920,35 +926,18 @@ void OrthoKey(PyMOLGlobals * G, unsigned char k, int x, int y, int mod)
       }
       break;
     case 127:                  /* delete */
-#if !defined(_PYMOL_OSX) || defined(_PYMOL_LIB)
-      /* this defined(_PYMOL_LIB) should really be for JyMOL, not all _PYMOL_LIB, AX? */
       if((!I->CurChar) || (I->CurChar == I->PromptChar) || !OrthoTextVisible(G)) {
         OrthoKeyControl(G, 4 + 64);
       } else {
-        if(I->CursorChar >= 0) {
-          if(I->CursorChar < I->CurChar)
-            I->CursorChar++;
-          if(I->CursorChar == I->CurChar)
-            I->CursorChar = -1;
-        }
-        if(I->CurChar > I->PromptChar) {
+        if(I->CursorChar > -1 && I->CursorChar < I->CurChar) {
           curLine = I->CurLine & OrthoSaveLines;
-          if(I->CursorChar >= 0) {
-            if(I->CursorChar > I->PromptChar) {
-              strcpy(buffer, I->Line[curLine] + I->CursorChar);
-              I->CursorChar--;
-              I->CurChar--;
-              strcpy(I->Line[curLine] + I->CursorChar, buffer);
-            }
-          } else {
-            I->CurChar--;
-            I->Line[curLine][I->CurChar] = 0;
-          }
+          strcpy(buffer, I->Line[curLine] + I->CursorChar + 1);
+          I->CurChar--;
+          strcpy(I->Line[curLine] + I->CursorChar, buffer);
         }
       }
       break;
     case 8:                    /* backspace */
-#endif
       if(I->CurChar > I->PromptChar) {
         curLine = I->CurLine & OrthoSaveLines;
         if(I->CursorChar >= 0) {
@@ -1007,6 +996,7 @@ void OrthoKey(PyMOLGlobals * G, unsigned char k, int x, int y, int mod)
             curLine = I->CurLine & OrthoSaveLines;
             strcpy(I->Line[curLine], buffer);
             I->CurChar = strlen(I->Line[curLine]);
+            I->CursorChar = -1;
           }
         }
       }
@@ -1133,7 +1123,7 @@ void OrthoAddOutput(PyMOLGlobals * G, const char *str)
   q = I->Line[curLine] + I->CurChar;
   cc = I->CurChar;
   while(*p) {
-    if(*p >= 32) {
+    if(*p != '\r' && *p != '\n') {
       cc++;
       wrap = SettingGetGlobal_b(G, cSetting_wrap_output);
 
@@ -1156,7 +1146,7 @@ void OrthoAddOutput(PyMOLGlobals * G, const char *str)
         curLine = I->CurLine & OrthoSaveLines;
       }
       *q++ = *p++;
-    } else if((*p == 13) || (*p == 10)) {
+    } else {
       *q = 0;
       I->CurChar = cc;
       OrthoNewLine(G, NULL, true);
@@ -1164,8 +1154,7 @@ void OrthoAddOutput(PyMOLGlobals * G, const char *str)
       curLine = I->CurLine & OrthoSaveLines;
       p++;
       cc = 0;
-    } else
-      p++;
+    }
   }
   *q = 0;
   I->CurChar = strlen(I->Line[curLine]);
@@ -1192,12 +1181,18 @@ void OrthoNewLine(PyMOLGlobals * G, const char *prompt, int crlf)
     OrthoFeedbackIn(G, I->Line[I->CurLine & OrthoSaveLines]);
   else
     OrthoFeedbackIn(G, " ");
-  if(Feedback(G, FB_Python, FB_Output)) {
-    if(crlf) {
-      printf("%s\n", I->Line[I->CurLine & OrthoSaveLines]);
-    } else {
+
+  bool do_print = Feedback(G, FB_Python, FB_Output);
+  bool do_print_with_escapes = false;
+
+  if (do_print) {
+    if (!do_print_with_escapes) {
       printf("%s", I->Line[I->CurLine & OrthoSaveLines]);
     }
+
+    if(crlf) {
+      putchar('\n');
+    }
     fflush(stdout);
   }
   /*        } */
@@ -1528,13 +1523,21 @@ void OrthoDoDraw(PyMOLGlobals * G, int render_mode)
     " OrthoDoDraw: entered.\n" ENDFD;
   if(G->HaveGUI && G->ValidContext) {
 
+#ifdef GL_FRAMEBUFFER_UNDEFINED
+    // prevents GL_INVALID_FRAMEBUFFER_OPERATION (0x0506) on macOS
+    // with external Monitor
+    if (glCheckFramebufferStatus &&
+        glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_UNDEFINED)
+      return;
+#endif
+
     if(Feedback(G, FB_OpenGL, FB_Debugging))
       PyMOLCheckOpenGLErr("OrthoDoDraw checkpoint 0");
 
     if(SettingGetGlobal_b(G, cSetting_internal_gui)) {
       switch (SettingGetGlobal_i(G, cSetting_internal_gui_mode)) {
       case 0:
-        rightSceneMargin = SettingGetGlobal_i(G, cSetting_internal_gui_width);
+        rightSceneMargin = DIP2PIXEL(SettingGetGlobal_i(G, cSetting_internal_gui_width));
         break;
       default:
         rightSceneMargin = 0;
@@ -1712,7 +1715,7 @@ void OrthoDoDraw(PyMOLGlobals * G, int render_mode)
         " OrthoDoDraw: drawing blocks...\n" ENDFD;
 
       if(SettingGetGlobal_b(G, cSetting_internal_gui)) {
-        int internal_gui_width = SettingGetGlobal_i(G, cSetting_internal_gui_width);
+        int internal_gui_width = DIP2PIXEL(SettingGetGlobal_i(G, cSetting_internal_gui_width));
         if(internal_gui_mode != 2) {
 	  if (generate_shader_cgo){
 	    CGOColor(orthoCGO, 0.3f, 0.3f, 0.3f);
@@ -2104,7 +2107,7 @@ static void OrthoLayoutPanel(PyMOLGlobals * G,
   COrtho *I = G->Ortho;
   Block *block = NULL;
 
-  int controlHeight = 20;
+  int controlHeight = DIP2PIXEL(20);
   int butModeHeight = ButModeGetHeight(G);
   int wizardHeight = I->WizardHeight;
 
@@ -2174,6 +2177,9 @@ void OrthoReshape(PyMOLGlobals * G, int width, int height, int force)
 {
   COrtho *I = G->Ortho;
 
+  if(!G->HaveGUI && width < 0)
+    return;
+
   Block *block = NULL;
   int sceneBottom, sceneRight = 0;
   int textBottom = 0;
@@ -2219,7 +2225,7 @@ void OrthoReshape(PyMOLGlobals * G, int width, int height, int force)
     else
       sceneBottom = textBottom;
 
-    internal_gui_width = SettingGetGlobal_i(G, cSetting_internal_gui_width);
+    internal_gui_width = DIP2PIXEL(SettingGetGlobal_i(G, cSetting_internal_gui_width));
     if(!SettingGetGlobal_b(G, cSetting_internal_gui)) {
       internal_gui_width = 0;
       sceneRight = 0;
@@ -2299,7 +2305,8 @@ void OrthoReshapeWizard(PyMOLGlobals * G, ov_size wizHeight)
 
   if(SettingGetGlobal_b(G, cSetting_internal_gui) > 0.0) {
     Block *block;
-    int internal_gui_width = SettingGetGlobal_i(G, cSetting_internal_gui_width);
+    int internal_gui_width = DIP2PIXEL(SettingGetGlobal_i(G, cSetting_internal_gui_width));
+
     OrthoLayoutPanel(G, 0, I->Width - internal_gui_width, I->TextBottom, 0);
 
     block = ExecutiveGetBlock(G);
