@@ -38,14 +38,10 @@ the initialization functions for these libraries on startup.
 
 /* END PROPRIETARY CODE SEGMENT */
 
-#ifndef _PYMOL_NOPY
 #ifdef _PYMOL_MINGW
 #define putenv _putenv
 #endif
-#include"os_python.h"
-#endif
 
-#include"os_std.h"
 #include"os_time.h"
 #include"os_unix.h"
 
@@ -64,6 +60,7 @@ the initialization functions for these libraries on startup.
 #include"PyMOLOptions.h"
 #include"PyMOL.h"
 #include "Lex.h"
+#include "Seeker.h"
 
 static int label_copy_text(char *dst, const char *src, int len, int max)
 {
@@ -119,31 +116,6 @@ static int label_next_token(WordType dst, const char **expr)
   return (q != dst);
 }
 
-/*
- * Get R/S label for SDF stereo enumeration
- */
-char convertStereoToChar(int stereo){
-  switch (stereo){
-    case 1: return 'S';
-    case 2: return 'R';
-    case 3: return '?';
-  }
-  return '\0';
-}
-
-/*
- * Get SDF stereo enumeration for R/S label
- */
-int convertCharToStereo(char stereo){
-  switch (stereo){
-    case 'S': case 's': return 1;
-    case 'R': case 'r': return 2;
-    case '?': return 3;
-  }
-  return 0;
-}
-
-
 int PLabelExprUsesVariable(PyMOLGlobals * G, const char *expr, const char *var)
 {
   char ch, quote = 0;
@@ -274,7 +246,7 @@ int PLabelAtomAlt(PyMOLGlobals * G, AtomInfoType * at, const char *model, const
           } else if(!strcmp(tok, "formal_charge")) {
             sprintf(buffer, "%d", at->formalCharge);
           } else if(!strcmp(tok, "stereo")) {
-	    sprintf(buffer, "%c", convertStereoToChar(at->stereo));
+            strcpy(buffer, AtomInfoGetStereoAsStr(at));
           } else if(!strcmp(tok, "color")) {
             sprintf(buffer, "%d", at->color);
           } else if(!strcmp(tok, "cartoon")) {
@@ -363,6 +335,27 @@ static PyObject *P_parser = NULL;
 static PyObject *P_main = NULL;
 static PyObject *P_vfont = NULL;
 
+/* module import helper */
+
+static
+PyObject * PImportModuleOrFatal(const char * name) {
+  PyObject * mod = PyImport_ImportModule(name);
+  if(!mod) {
+    fprintf(stderr, "PyMOL-Error: can't find '%s'\n", name);
+    exit(EXIT_FAILURE);
+  }
+  return mod;
+}
+
+static
+PyObject * PGetAttrOrFatal(PyObject * o, const char * name) {
+  PyObject * attr = PyObject_GetAttrString(o, name);
+  if(!attr) {
+    fprintf(stderr, "PyMOL-Error: can't find '%s'\n", name);
+    exit(EXIT_FAILURE);
+  }
+  return attr;
+}
 
 /* used elsewhere */
 
@@ -375,9 +368,13 @@ PyObject *P_setting = NULL;     /* okay as global -- just used for names */
 static PyMappingMethods wrapperMappingMethods, settingMappingMethods;
 static PyTypeObject Wrapper_Type = {
   PyVarObject_HEAD_INIT(NULL, 0)
+  "wrapper.Wrapper",            /* tp_name */
+  0,                            /* tp_basicsize */
 };
 static PyTypeObject settingWrapper_Type = {
   PyVarObject_HEAD_INIT(NULL, 0)
+  "wrapper.SettingWrapper",     /* tp_name */
+  0,                            /* tp_basicsize */
 };
 
 /*
@@ -581,7 +578,8 @@ PyObject * WrapperObjectSubScript(PyObject *obj, PyObject *key){
   if (!check_wrapper_scope(wobj))
     return NULL;
 
-  char *aprop;
+  PyMOLGlobals * G = wobj->G;
+  const char *aprop;
   AtomPropertyInfo *ap;
   PyObject *ret = NULL;
   bool borrowed = false;
@@ -667,14 +665,7 @@ PyObject * WrapperObjectSubScript(PyObject *obj, PyObject *key){
           "'properties/p' not supported in Open-Source PyMOL");
       break;
     case cPType_state:
-      {
-	if (wobj->idx >= 0){
-	  ret = PyInt_FromLong((long)wobj->state);
-	} else {
-          PyErr_SetString(PyExc_NameError,
-              "'state' only available in iterate_state and alter_state");
-	}
-      }
+      ret = PyInt_FromLong((long)wobj->state);
       break;
     default:
       switch (ap->id) {
@@ -687,10 +678,17 @@ PyObject * WrapperObjectSubScript(PyObject *obj, PyObject *key){
         break;
       case ATOM_PROP_STEREO:
         {
-          char mmstereotype[] = {convertStereoToChar(wobj->atomInfo->stereo), '\0'};
+          auto mmstereotype = AtomInfoGetStereoAsStr(wobj->atomInfo);
           ret = PyString_FromString(mmstereotype);
         }
         break;
+      case ATOM_PROP_ONELETTER:
+        {
+          const char * st = LexStr(G, wobj->atomInfo->resn);
+          char abbr[2] = {SeekerGetAbbr(G, st, 'O', 'X'), 0};
+          ret = PyString_FromString(abbr);
+        }
+        break;
       default:
         PyErr_SetString(PyExc_SystemError, "unhandled atom property type");
       }
@@ -698,7 +696,11 @@ PyObject * WrapperObjectSubScript(PyObject *obj, PyObject *key){
   } else {
     /* if not an atom property, check if local variable in dict */
     ret = PyDict_GetItem(wobj->dict, key);
-    borrowed = true;
+    if (ret) {
+      borrowed = true;
+    } else {
+      PyErr_SetNone(PyExc_KeyError);
+    }
   }
 
   if (borrowed)
@@ -741,19 +743,13 @@ int WrapperObjectAssignSubScript(PyObject *obj, PyObject *key, PyObject *val){
           PConvPyObjectToFloat(val, v);
           return 0;
         }
-
-        if (ap->id != ATOM_PROP_FLAGS) {
-          PyErr_SetString(PyExc_TypeError,
-              "only x/y/z/flags can be modified in alter_state");
-	  return -1;
-	}
       }
 
       switch (ap->Ptype){
       case cPType_string:
 	{
           PyObject *valobj = PyObject_Str(val);
-	  char *valstr = PyString_AS_STRING(valobj);
+	  const char *valstr = PyString_AS_STRING(valobj);
 	  char *dest = (char*)(((char*)wobj->atomInfo) + ap->offset);
 	  if (strlen(valstr) > ap->maxlen){
 	    strncpy(dest, valstr, ap->maxlen);
@@ -791,7 +787,7 @@ int WrapperObjectAssignSubScript(PyObject *obj, PyObject *key, PyObject *val){
           auto dest = reinterpret_cast<lexidx_t*>
             (((char*)wobj->atomInfo) + ap->offset);
           PyObject *valobj = PyObject_Str(val);
-	  char *valstr = PyString_AS_STRING(valobj);
+	  const char *valstr = PyString_AS_STRING(valobj);
 	  LexDec(wobj->G, *dest);
 	  *dest = LexIdx(wobj->G, valstr);
           Py_DECREF(valobj);
@@ -807,7 +803,7 @@ int WrapperObjectAssignSubScript(PyObject *obj, PyObject *key, PyObject *val){
       case cPType_char_as_type:
 	{
           PyObject *valobj = PyObject_Str(val);
-          char *valstr = PyString_AS_STRING(valobj);
+          const char *valstr = PyString_AS_STRING(valobj);
           wobj->atomInfo->hetatm = ((valstr[0] == 'h') || (valstr[0] == 'H'));
           Py_DECREF(valobj);
 	  changed = true;
@@ -816,7 +812,7 @@ int WrapperObjectAssignSubScript(PyObject *obj, PyObject *key, PyObject *val){
       case cPType_int_custom_type:
 	{
           PyObject *valobj = PyObject_Str(val);
-	  char *valstr = PyString_AS_STRING(valobj);
+	  const char *valstr = PyString_AS_STRING(valobj);
 	  int *dest = (int*)(((char*)wobj->atomInfo) + ap->offset);
 	  if (valstr[0] == '?'){
 	    *dest = cAtomInfoNoType;
@@ -847,7 +843,7 @@ int WrapperObjectAssignSubScript(PyObject *obj, PyObject *key, PyObject *val){
           {
             PyObject *valobj = PyObject_Str(val);
             const char *valstr = PyString_AS_STRING(valobj);
-            wobj->atomInfo->stereo = convertCharToStereo(valstr[0]);
+            AtomInfoSetStereo(wobj->atomInfo, valstr);
             Py_DECREF(valobj);
           }
           break;
@@ -859,6 +855,8 @@ int WrapperObjectAssignSubScript(PyObject *obj, PyObject *key, PyObject *val){
       if (changed){
 	switch (ap->id){
 	case ATOM_PROP_ELEM:
+	  wobj->atomInfo->protons = 0;
+	  wobj->atomInfo->vdw = 0;
 	  AtomInfoAssignParameters(wobj->G, wobj->atomInfo);
 	  break;
 	case ATOM_PROP_RESV:
@@ -954,9 +952,7 @@ PyObject *PGetFontDict(PyMOLGlobals * G, float size, int face, int style)
   PyObject *result = NULL;
 
   if(!P_vfont) {
-    PRunStringModule(G, "import pymol.vfont\n");
-    P_vfont = PyDict_GetItemString(P_pymol_dict, "vfont");
-    Py_XINCREF(P_vfont);
+    P_vfont = PyImport_ImportModule("pymol.vfont");
   }
   if(!P_vfont) {
     PRINTFB(G, FB_Python, FB_Errors)
@@ -971,7 +967,7 @@ int PComplete(PyMOLGlobals * G, char *str, int buf_size)
 {
   int ret = false;
   PyObject *result;
-  char *st2;
+  const char *st2;
   PBlockAndUnlockAPI(G);
   if(G->P_inst->complete) {
     result = PYOBJECT_CALLFUNCTION(G->P_inst->complete, "s", str);
@@ -1236,6 +1232,7 @@ void PSleep(PyMOLGlobals * G, int usec)
 }
 
 static PyObject *PCatchWrite(PyObject * self, PyObject * args);
+static PyObject *PCatch_install(PyObject * self, PyObject * args);
 
 void my_interrupt(int a)
 {
@@ -1282,24 +1279,8 @@ int PAlterAtom(PyMOLGlobals * G,
                ObjectMolecule *obj, CoordSet *cs, PyCodeObject *expr_co, int read_only,
                int atm, PyObject * space)
 {
-  int result = true;
-
-  G->P_inst->wrapperObject->obj = obj;
-  G->P_inst->wrapperObject->cs = cs;
-  G->P_inst->wrapperObject->atomInfo = obj->AtomInfo + atm;
-  G->P_inst->wrapperObject->atm = atm;
-  G->P_inst->wrapperObject->idx = -1;
-  G->P_inst->wrapperObject->read_only = read_only;
-  G->P_inst->wrapperObject->state = -1;
-
-  PXDecRef(PyEval_EvalCode(expr_co, space, (PyObject*)G->P_inst->wrapperObject));
-  WrapperObjectReset(G->P_inst->wrapperObject);
-
-  if(PyErr_Occurred()) {
-    PyErr_Print();
-    result = false;
-  }
-  return result;
+  int state = (obj->DiscreteFlag ? obj->AtomInfo[atm].discrete_state : 0) - 1;
+  return PAlterAtomState(G, expr_co, read_only, obj, cs, atm, /* idx */ -1, state, space);
 }
 
 /*
@@ -1331,7 +1312,12 @@ int PLabelAtom(PyMOLGlobals * G, ObjectMolecule *obj, CoordSet *cs, PyCodeObject
   G->P_inst->wrapperObject->atm = atm;
   G->P_inst->wrapperObject->idx = -1;
   G->P_inst->wrapperObject->read_only = true;
-  G->P_inst->wrapperObject->state = -1;
+
+  if (obj->DiscreteFlag) {
+    G->P_inst->wrapperObject->state = obj->AtomInfo[atm].discrete_state;
+  } else {
+    G->P_inst->wrapperObject->state = 0;
+  }
 
   if (!expr_co){
     // unsetting label
@@ -1579,7 +1565,7 @@ static int IsSecurityRequired()
 void PSetupEmbedded(PyMOLGlobals * G, int argc, char **argv)
 {
   /* This routine is called if we are running with an embedded Python interpreter */
-  PyObject *args, *pymol;
+  PyObject *args;
 
   /* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */
 #ifdef WIN32
@@ -1944,8 +1930,6 @@ void PSetupEmbedded(PyMOLGlobals * G, int argc, char **argv)
   /* ultimate fallback -- try using the current working directory */
   PyRun_SimpleString
     ("if 'PYMOL_PATH' not in os.environ: os.environ['PYMOL_PATH']=os.getcwd()\n");
-  PyRun_SimpleString
-    ("if (os.environ['PYMOL_PATH']+'/modules') not in sys.path: sys.path.insert(0,os.environ['PYMOL_PATH']+'/modules')\n");
 #endif
   /* END PROPRIETARY CODE SEGMENT */
 
@@ -1953,9 +1937,6 @@ void PSetupEmbedded(PyMOLGlobals * G, int argc, char **argv)
   if(!P_main)
     ErrFatal(G, "PyMOL", "can't find '__main__'");
 
-  /* inform PyMOL's other half that we're launching embedded-style */
-  PyObject_SetAttrString(P_main, "pymol_launch", PyInt_FromLong(4));
-
   args = PConvStringListToPyList(argc, argv);   /* prepare our argument list */
   if(!args)
     ErrFatal(G, "PyMOL", "can't process arguments.");
@@ -1966,18 +1947,11 @@ void PSetupEmbedded(PyMOLGlobals * G, int argc, char **argv)
     ("import __main__\nif not hasattr(sys,'argv'): sys.argv=__main__.pymol_argv");
 
   PyRun_SimpleString("if (os.environ['PYMOL_PATH']+'/modules') not in sys.path: sys.path.insert(0,os.environ['PYMOL_PATH']+'/modules')\n");     /* needed for semistatic pymol */
-
-  PyRun_SimpleString("import pymol");   /* create the global PyMOL namespace */
-
-  pymol = PyImport_AddModule("pymol");  /* get it */
-  if(!pymol)
-    ErrFatal(G, "PyMOL", "can't find module 'pymol'");
-
 }
 
 void PConvertOptions(CPyMOLOptions * rec, PyObject * options)
 {
-  char *load_str;
+  const char *load_str;
 
   rec->pmgui = !PyInt_AsLong(PyObject_GetAttrString(options, "no_gui"));
   rec->internal_gui = PyInt_AsLong(PyObject_GetAttrString(options, "internal_gui"));
@@ -2035,27 +2009,14 @@ void PGetOptions(CPyMOLOptions * rec)
 {
   PyObject *pymol, *invocation, *options;
 
-  pymol = PyImport_AddModule("pymol");  /* borrowed reference!!! */
-  if(!pymol) {
-    fprintf(stderr, "PyMOL-ERROR: can't find module 'pymol'");
-    exit(EXIT_FAILURE);
-  }
-
-  invocation = PyObject_GetAttrString(pymol, "invocation");     /* get a handle to the invocation module */
-  if(!invocation) {
-    fprintf(stderr, "PyMOL-ERROR: can't find module 'invocation'");
-    exit(EXIT_FAILURE);
-  }
-
-  options = PyObject_GetAttrString(invocation, "options");
-  if(!options) {
-    fprintf(stderr, "PyMOL-ERROR: can't get 'invocation.options'.");
-    exit(EXIT_FAILURE);
-  }
+  pymol = PImportModuleOrFatal("pymol");
+  invocation = PGetAttrOrFatal(pymol, "invocation");     /* get a handle to the invocation module */
+  options = PGetAttrOrFatal(invocation, "options");
 
   PConvertOptions(rec, options);
   Py_XDECREF(invocation);
   Py_XDECREF(options);
+  Py_XDECREF(pymol);
 }
 
 void PRunStringModule(PyMOLGlobals * G, const char *str)
@@ -2077,8 +2038,6 @@ void WrapperObjectReset(WrapperObject *wo){
 
 void PInit(PyMOLGlobals * G, int global_instance)
 {
-  PyObject *sys, *pcatch;
-
 #ifdef PYMOL_NEW_THREADS
   PyEval_InitThreads();
 #endif
@@ -2116,15 +2075,18 @@ void PInit(PyMOLGlobals * G, int global_instance)
   /* initialize PyOpenGL */
 #endif
 
-  if(global_instance) {
+#if PY_MAJOR_VERSION < 3
+  // Support implicit utf-8 encoding (important for labeling!)
+  //   str(u"...unicode...") -> b"...utf-8..."
+  //   PyString_AsString(unicodeobj) -> "...utf-8..."
+  PyUnicode_SetDefaultEncoding("utf-8");
+#endif
+
+  if(true /* global_instance */) {
     PCatchInit();               /* setup standard-output catch routine */
   }
 
-  /* assumes that pymol module has been loaded via Python or PyRun_SimpleString */
-
-  P_pymol = PyImport_AddModule("pymol");        /* get it */
-  if(!P_pymol)
-    ErrFatal(G, "PyMOL", "can't find module 'pymol'");
+  P_pymol = PImportModuleOrFatal("pymol");
   P_pymol_dict = PyModule_GetDict(P_pymol);
   Py_XINCREF(P_pymol_dict);
   if(!P_pymol_dict)
@@ -2144,40 +2106,14 @@ void PInit(PyMOLGlobals * G, int global_instance)
   }
 
   {
-    G->P_inst->exec = PyDict_GetItemString(P_pymol_dict, "exec_str");
-    Py_XINCREF(G->P_inst->exec);
-    if(!G->P_inst->exec)
-      ErrFatal(G, "PyMOL", "can't find 'pymol.exec_str()'");
-
-    sys = PyDict_GetItemString(P_pymol_dict, "sys");
-    Py_XINCREF(sys);
-    if(!sys)
-      ErrFatal(G, "PyMOL", "can't find 'pymol.sys'");
+    G->P_inst->exec = PGetAttrOrFatal(P_pymol, "exec_str");
 
     if(global_instance) {
-
-      /* implies global singleton pymol, so set up the global handle */
-      PyDict_SetItemString(P_pymol_dict, "_COb",
-                           PyCObject_FromVoidPtr((void *) &SingletonPyMOLGlobals, NULL));
-
-      pcatch = PyImport_ImportModule("pcatch");
-      if(!pcatch)
-        ErrFatal(G, "PyMOL", "can't find module 'pcatch'");
-      PyObject_SetAttrString(sys, "stdout", pcatch);
-      PyObject_SetAttrString(sys, "stderr", pcatch);
+      PCatch_install(NULL, NULL);
     }
 
-    PRunStringModule(G, "import traceback\n");
-    P_traceback = PyDict_GetItemString(P_pymol_dict, "traceback");
-    Py_XINCREF(P_traceback);
-    if(!P_traceback)
-      ErrFatal(G, "PyMOL", "can't find 'traceback'");
-
-    PRunStringModule(G, "import pymol.cmd\n");
-    P_cmd = PyDict_GetItemString(P_pymol_dict, "cmd");
-    Py_XINCREF(P_cmd);
-    if(!P_cmd)
-      ErrFatal(G, "PyMOL", "can't find 'cmd'");
+    P_traceback = PImportModuleOrFatal("traceback");
+    P_cmd = PImportModuleOrFatal("pymol.cmd");
 
     if(global_instance) {
       /* implies global singleton pymol, so set up the global handle */
@@ -2189,109 +2125,46 @@ void PInit(PyMOLGlobals * G, int global_instance)
       G->P_inst->cmd = P_cmd;
     }
 
-    PyObject_SetAttrString(G->P_inst->cmd, "_pymol", G->P_inst->obj);
-
     /* right now, all locks are global -- eventually some of these may
        become instance-specific in order to improve concurrency */
 
-    G->P_inst->lock = PyObject_GetAttrString(G->P_inst->cmd, "lock");
-    if(!G->P_inst->lock)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.lock()'");
-
-    G->P_inst->lock_attempt = PyObject_GetAttrString(G->P_inst->cmd, "lock_attempt");
-    if(!G->P_inst->lock_attempt)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.lock_attempt()'");
-
-    G->P_inst->unlock = PyObject_GetAttrString(G->P_inst->cmd, "unlock");
-    if(!G->P_inst->unlock)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.unlock()'");
-
-    G->P_inst->lock_c = PyObject_GetAttrString(G->P_inst->cmd, "lock_c");
-    if(!G->P_inst->lock_c)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.lock_c()'");
-
-    G->P_inst->unlock_c = PyObject_GetAttrString(G->P_inst->cmd, "unlock_c");
-    if(!G->P_inst->unlock_c)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.unlock_c()'");
-
-    G->P_inst->lock_status = PyObject_GetAttrString(G->P_inst->cmd, "lock_status");
-    if(!G->P_inst->lock_status)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.lock_status()'");
-
+    G->P_inst->lock = PGetAttrOrFatal(G->P_inst->cmd, "lock");
+    G->P_inst->lock_attempt = PGetAttrOrFatal(G->P_inst->cmd, "lock_attempt");
+    G->P_inst->unlock = PGetAttrOrFatal(G->P_inst->cmd, "unlock");
+    G->P_inst->lock_c = PGetAttrOrFatal(G->P_inst->cmd, "lock_c");
+    G->P_inst->unlock_c = PGetAttrOrFatal(G->P_inst->cmd, "unlock_c");
+    G->P_inst->lock_status = PGetAttrOrFatal(G->P_inst->cmd, "lock_status");
     G->P_inst->lock_status_attempt =
-      PyObject_GetAttrString(G->P_inst->cmd, "lock_status_attempt");
-    if(!G->P_inst->lock_status_attempt)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.lock_status_attempt()'");
-
-    G->P_inst->unlock_status = PyObject_GetAttrString(G->P_inst->cmd, "unlock_status");
-    if(!G->P_inst->unlock_status)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.unlock_status()'");
-
-    G->P_inst->lock_glut = PyObject_GetAttrString(G->P_inst->cmd, "lock_glut");
-    if(!G->P_inst->lock_glut)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.lock_glut()'");
-
-    G->P_inst->unlock_glut = PyObject_GetAttrString(G->P_inst->cmd, "unlock_glut");
-    if(!G->P_inst->unlock_glut)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.unlock_glut()'");
+      PGetAttrOrFatal(G->P_inst->cmd, "lock_status_attempt");
+    G->P_inst->unlock_status = PGetAttrOrFatal(G->P_inst->cmd, "unlock_status");
+    G->P_inst->lock_glut = PGetAttrOrFatal(G->P_inst->cmd, "lock_glut");
+    G->P_inst->unlock_glut = PGetAttrOrFatal(G->P_inst->cmd, "unlock_glut");
 
     /* 'do' command */
 
-    G->P_inst->cmd_do = PyObject_GetAttrString(G->P_inst->cmd, "do");
-    if(!G->P_inst->cmd_do)
-      ErrFatal(G, "PyMOL", "can't find 'cmd.do()'");
+    G->P_inst->cmd_do = PGetAttrOrFatal(G->P_inst->cmd, "do");
 
     /* cache */
     G->P_inst->cache = PyObject_GetAttrString(G->P_inst->obj, "_cache");
 
     /* invariant stuff */
 
-    PRunStringModule(G, "import pymol.menu\n");
-    P_menu = PyDict_GetItemString(P_pymol_dict, "menu");
-    Py_XINCREF(P_menu);
-    if(!P_menu)
-      ErrFatal(G, "PyMOL", "can't find module 'menu'");
-
-    PRunStringModule(G, "import pymol.setting\n");
-    P_setting = PyDict_GetItemString(P_pymol_dict, "setting");
-    Py_XINCREF(P_setting);
-    if(!P_setting)
-      ErrFatal(G, "PyMOL", "can't find module 'setting'");
-
-    PRunStringModule(G, "import pymol.povray\n");
-    P_povray = PyDict_GetItemString(P_pymol_dict, "povray");
-    Py_XINCREF(P_povray);
-    if(!P_povray)
-      ErrFatal(G, "PyMOL", "can't find module 'povray'");
+    P_menu = PImportModuleOrFatal("pymol.menu");
+    P_setting = PImportModuleOrFatal("pymol.setting");
+    P_povray = PImportModuleOrFatal("pymol.povray");
 
 #ifdef _PYMOL_XRAY
-    PRunStringModule(G, "import pymol.xray\n");
-    P_xray = PyDict_GetItemString(P_pymol_dict, "xray");
-    Py_XINCREF(P_xray);
-    if(!P_xray)
-      ErrFatal(G, "PyMOL", "can't find module 'xray'");
+    P_xray = PImportModuleOrFatal("pymol.xray");
 #endif
 
     /* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */
 #ifdef WIN32
-    PRunStringModule(G, "import time\n");
-    P_time = PyDict_GetItemString(P_pymol_dict, "time");
-    Py_XINCREF(P_time);
-    if(!P_time)
-      ErrFatal(G, "PyMOL", "can't find module 'time'");
-
-    P_sleep = PyObject_GetAttrString(P_time, "sleep");
-    Py_XINCREF(P_sleep);
-    if(!P_sleep)
-      ErrFatal(G, "PyMOL", "can't find 'time.sleep()'");
+    P_time = PImportModuleOrFatal("time");
+    P_sleep = PGetAttrOrFatal(P_time, "sleep");
 #endif
     /* END PROPRIETARY CODE SEGMENT */
 
-    PRunStringModule(G, "import pymol.parser\n");
-    P_parser = PyDict_GetItemString(P_pymol_dict, "parser");
-    Py_XINCREF(P_parser);
-    if(!P_parser)
-      ErrFatal(G, "PyMOL", "can't find module 'parser'");
+    P_parser = PImportModuleOrFatal("pymol.parser");
 
     {
       PyObject *fn_closure = PyObject_GetAttrString(P_parser, "new_parse_closure");
@@ -2309,31 +2182,17 @@ void PInit(PyMOLGlobals * G, int global_instance)
         ErrFatal(G, "PyMOL", "can't create 'complete' function closure");
     }
 
-    PRunStringModule(G, "import chempy");
-    P_chempy = PyDict_GetItemString(P_pymol_dict, "chempy");
-    Py_XINCREF(P_chempy);
-    if(!P_chempy)
-      ErrFatal(G, "PyMOL", "can't find 'chempy'");
+    {
+      PyObject *fn_closure = PGetAttrOrFatal(P_pymol, "_colortype");
+      G->P_inst->colortype = PYOBJECT_CALLFUNCTION(fn_closure, "O", G->P_inst->cmd);
+      PXDecRef(fn_closure);
+    }
 
-    PRunStringModule(G, "from chempy.bonds import bonds");      /* load bond dictionary */
-
-    PRunStringModule(G, "from chempy import models");
-    P_models = PyDict_GetItemString(P_pymol_dict, "models");
-    Py_XINCREF(P_models);
-    if(!P_models)
-      ErrFatal(G, "PyMOL", "can't find 'chempy.models'");
-
-    PRunStringModule(G, "import pymol.util\n");
-    PRunStringModule(G, "import pymol.preset\n");
-    PRunStringModule(G, "import pymol.contrib\n");
-
-    PRunStringModule(G, "import string\n");
+    P_chempy = PImportModuleOrFatal("chempy");
+    P_models = PImportModuleOrFatal("chempy.models");
 
     /* backwards compatibility */
 
-    PRunStringModule(G, "pm = cmd\n");
-    PRunStringModule(G, "pmu = util\n");
-
     PRunStringModule(G, "glutThread = thread.get_ident()");
 
     P_glut_thread_id = PyThread_get_thread_ident();
@@ -2344,18 +2203,7 @@ void PInit(PyMOLGlobals * G, int global_instance)
     }
 #endif
 
-    /* required environment variables */
-
-    PyRun_SimpleString("import os");
-
-    PyRun_SimpleString
-      ("if 'PYMOL_DATA' not in os.environ: os.environ['PYMOL_DATA']=os.environ['PYMOL_PATH']+'/data'");
-    PyRun_SimpleString("os.environ['TUT']=os.environ['PYMOL_DATA']+'/tut'");
-
-    PyRun_SimpleString
-      ("if 'PYMOL_SCRIPTS' not in os.environ: os.environ['PYMOL_SCRIPTS']=os.environ['PYMOL_PATH']+'/scripts'");
-
-    Wrapper_Type.tp_name = "wrapper.Wrapper";
+  if (!Wrapper_Type.tp_basicsize) {
     Wrapper_Type.tp_basicsize = sizeof(WrapperObject);
     Wrapper_Type.tp_flags = Py_TPFLAGS_DEFAULT;
     wrapperMappingMethods.mp_length = NULL;
@@ -2363,7 +2211,6 @@ void PInit(PyMOLGlobals * G, int global_instance)
     wrapperMappingMethods.mp_ass_subscript = &WrapperObjectAssignSubScript;
     Wrapper_Type.tp_as_mapping = &wrapperMappingMethods;
     
-    settingWrapper_Type.tp_name = "wrapper.SettingWrapper";
     settingWrapper_Type.tp_basicsize = sizeof(SettingPropertyWrapperObject);
     settingWrapper_Type.tp_flags = Py_TPFLAGS_DEFAULT;
     settingWrapper_Type.tp_iter = &SettingWrapperObjectIter;
@@ -2383,6 +2230,8 @@ void PInit(PyMOLGlobals * G, int global_instance)
     }
     Py_INCREF(&Wrapper_Type);
     Py_INCREF(&settingWrapper_Type);
+  }
+
     G->P_inst->wrapperObject = (WrapperObject *)PyType_GenericNew(&Wrapper_Type, Py_None, Py_None);
     G->P_inst->wrapperObject->G = G;
     G->P_inst->wrapperObject->dict = PyDict_New();
@@ -2391,6 +2240,14 @@ void PInit(PyMOLGlobals * G, int global_instance)
     Py_INCREF(G->P_inst->wrapperObject);
     Py_INCREF(G->P_inst->settingWrapperObject);
   }
+
+#ifdef _PYMOL_NO_MSGPACKC
+  // fallback MMTF support
+  PyRun_SimpleString(
+      "import pymol.importing;"
+      "pymol.importing.loadfunctions.setdefault('mmtf',"
+      "pymol.importing.load_mmtf)");
+#endif
 }
 
 int PPovrayRender(PyMOLGlobals * G, const char *header, const char *inp, const char *file, int width,
@@ -2420,27 +2277,40 @@ void PSGIStereo(PyMOLGlobals * G, int flag)
     PUnblock(G);
 }
 
-void PFree(void)
+void PFree(PyMOLGlobals * G)
 {
+  PXDecRef(G->P_inst->parse);
+  PXDecRef(G->P_inst->complete);
+  PXDecRef(G->P_inst->colortype);
 }
 
 void PExit(PyMOLGlobals * G, int code)
 {
   ExecutiveDelete(G, "all");
   PBlock(G);
+
+  PyMOL_PushValidContext(G->PyMOL);
+  PyMOL_Stop(G->PyMOL);
+  PyMOL_PopValidContext(G->PyMOL);
+
 #ifndef _PYMOL_NO_MAIN
   if(G->Main) {
     MainFree();
   }
 #endif
 
+  PyMOL_Free(G->PyMOL);
+
+#if 1
   /* we're having trouble with threading errors after calling Py_Exit,
      so for the time being, let's just take the process down at this
      point, instead of allowing PyExit to be called. */
 
   exit(code);
+#else
 
   Py_Exit(code);
+#endif
 }
 
 void PParse(PyMOLGlobals * G, const char *str)
@@ -2876,7 +2746,7 @@ static PyObject *PCatchWritelines(PyObject * self, PyObject * args)
       for(i = 0; i < len; i++) {
         PyObject *obj = PySequence_GetItem(seq, i);
         if(obj && PyString_Check(obj)) {
-          char *str = PyString_AsString(obj);
+          const char *str = PyString_AsString(obj);
           if(SingletonPyMOLGlobals) {
             if(Feedback(SingletonPyMOLGlobals, FB_Python, FB_Output)) {
               OrthoAddOutput(SingletonPyMOLGlobals, str);
@@ -2902,11 +2772,23 @@ static PyObject *PCatchIsAtty(PyObject * self, PyObject * args)
   Py_RETURN_FALSE;
 }
 
+static PyObject *PCatch_install(PyObject * self, PyObject * args)
+{
+  PyRun_SimpleString(
+      "import sys, pcatch\n"
+      "if sys.stdout is not pcatch:"
+      "pcatch.closed = False;"
+      "pcatch.encoding = 'UTF-8';"
+      "sys.stderr = sys.stdout = pcatch");
+  return PConvAutoNone(Py_None);
+}
+
 static PyMethodDef PCatch_methods[] = {
   {"writelines", PCatchWritelines, METH_VARARGS},
   {"write", PCatchWrite, METH_VARARGS},
   {"flush", PCatchFlush, METH_VARARGS},
   {"isatty", PCatchIsAtty, METH_VARARGS}, // called by pip.main(["install", "..."])
+  {"_install", PCatch_install, METH_VARARGS},
   {NULL, NULL}                  /* sentinel */
 };
 
