@@ -36,6 +36,7 @@ Z* -------------------------------------------------------------------
 
 #include"Executive.h"
 #include"CGO.h"
+#include"Selector.h"
 
 int ObjectGetNFrames(CObject * I);
 
@@ -661,6 +662,28 @@ void ObjectMakeValidName(char *name)
   }
 }
 
+/*
+ * Strip invalid characters from `name`, and if `name` equals a reserved
+ * selection keyword, then also append an underscore.
+ */
+void ObjectMakeValidName(PyMOLGlobals * G, char *name)
+{
+  ObjectMakeValidName(name);
+
+  if (SelectorNameIsKeyword(G, name)) {
+    PRINTFB(G, FB_Executive, FB_Warnings)
+      " Warning: '%s' is a reserved keyword, appending underscore\n", name
+      ENDFB(G);
+    strcat(name, "_");
+  } else if (strcmp(name, "protein") == 0 || strcmp(name, "nucleic") == 0) {
+    // Warn the user if "protein" or "nucleic" are used as names, but
+    // don't modify the name (yet).
+    PRINTFB(G, FB_Executive, FB_Warnings)
+      " Warning: '%s' may become a reserved selection keyword in the future\n", name
+      ENDFB(G);
+  }
+}
+
 int ObjectGetCurrentState(CObject * I, int ignore_all_states)
 {
   // the previous implementation (up to PyMOL 1.7.6) ignored
@@ -1090,10 +1113,24 @@ void ObjectToggleRepVis(CObject * I, int rep)
 
 
 /*========================================================================*/
-void ObjectSetRepVis(CObject * I, int rep, int state)
+void ObjectSetRepVisMask(CObject * I, int repmask, int value)
 {
-  if((rep >= 0) && (rep < cRepCnt))
-    SET_BIT_TO(I->visRep, rep, state);
+  switch (value) {
+    case cVis_HIDE:
+      I->visRep &= ~repmask;
+      break;
+    case cVis_SHOW:
+      I->visRep |= repmask;
+      break;
+    case cVis_AS:
+      I->visRep = repmask;
+      break;
+    case cVis_TOGGLE:
+      I->visRep ^= repmask;
+      break;
+    default:
+      printf("error: invalid value: %d\n", value);
+  }
 }
 
 
@@ -1102,7 +1139,7 @@ void ObjectSetName(CObject * I, const char *name)
 {
   UtilNCopy(I->Name, name, WordLength);
   if(SettingGetGlobal_b(I->G, cSetting_validate_object_names))
-    ObjectMakeValidName(I->Name);
+    ObjectMakeValidName(I->G, I->Name);
 }
 
 
