@@ -265,7 +265,11 @@ PyObject *SettingUniqueGetPyObject(PyMOLGlobals * G, int unique_id, int index)
     case cSetting_float:
       return CPythonVal_New_Float(val_f);
     case cSetting_color:
-      ptr_3f = ColorGet(G, val_i);
+#ifdef _PYMOL_NOPY
+      return CPythonVal_New_Integer(val_i);
+#else
+      return PYOBJECT_CALLFUNCTION(G->P_inst->colortype, "i", val_i);
+#endif
     case cSetting_float3:
       {
         PyObject *result = PyTuple_New(3);
@@ -623,6 +627,7 @@ static bool is_session_blacklisted(int index) {
   case cSetting_cgo_shader_ub_color:
   case cSetting_cgo_shader_ub_flags:
   case cSetting_cgo_shader_ub_normal:
+  case cSetting_colored_feedback:
   case cSetting_cylinder_shader_ff_workaround:
   case cSetting_defer_updates:
   case cSetting_fast_idle:
@@ -636,6 +641,7 @@ static bool is_session_blacklisted(int index) {
   case cSetting_nb_spheres_use_shader:
   case cSetting_no_idle:
   case cSetting_nvidia_bugs:
+  case cSetting_presentation:
   case cSetting_precomputed_lighting:
   case cSetting_render_as_cylinders:
   case cSetting_security:
@@ -657,6 +663,8 @@ static bool is_session_blacklisted(int index) {
   case cSetting_trilines:
   case cSetting_use_geometry_shaders:
   case cSetting_use_shaders:
+  case cSetting_pick32bit:
+  case cSetting_display_scale_factor:
 #ifdef _PYMOL_IOS
   case cSetting_cgo_sphere_quality:
   case cSetting_dynamic_measures:
@@ -863,9 +871,6 @@ int SettingSetGlobalsFromPyList(PyMOLGlobals * G, PyObject * list)
 
   /* restore the following settings  */
 
-  if(G->Option->presentation) {
-    SettingSet_b(I, cSetting_presentation, 1);
-  }
   if(G->Option->no_quit) {
     SettingSet_b(I, cSetting_presentation_auto_quit, 0);
   }
@@ -970,7 +975,7 @@ static int set_list(CSetting * I, PyObject * list)
     int val_i;
     float val_f;
     float val_3f[3];
-    char * val_s;
+    const char * val_s;
   };
 
   if (list == NULL || CPythonVal_IsNone(list))
@@ -1941,7 +1946,12 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
     break;
   case cSetting_light_count:
   case cSetting_spec_count:
+    CShaderMgr_Set_Reload_Bits(G, RELOAD_SHADERS_FOR_LIGHTING);
+    break;
   case cSetting_pick_shading:
+    if (SettingGetGlobal_i(G, cSetting_surface_color_smoothing)) {
+      ExecutiveInvalidateRep(G, inv_sele, cRepSurface, cRepInvColor);
+    }
     CShaderMgr_Set_Reload_Bits(G, RELOAD_SHADERS_FOR_LIGHTING);
   case cSetting_light:
   case cSetting_light2:
@@ -2782,6 +2792,7 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
   case cSetting_bg_image_linear:
   case cSetting_bg_image_tilesize:
   case cSetting_chromadepth:
+  case cSetting_colored_feedback:
   case cSetting_dash_transparency:
   case cSetting_label_bg_color:
   case cSetting_label_bg_outline:
@@ -2808,10 +2819,26 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
   case cSetting_suspend_undo:
   case cSetting_use_geometry_shaders:
   case cSetting_volume_mode:
+  case cSetting_surface_smooth_edges:
     PRINTFB(G, FB_Setting, FB_Warnings)
       " Setting-Warning: %s is not supported in Open-Source version of PyMOL\n",
       SettingInfo[index].name
       ENDFB(G);
+    break;
+  case cSetting_display_scale_factor:
+  {
+    int scaleFactor = SettingGetGlobal_i(G, cSetting_display_scale_factor);
+    if (scaleFactor > 0) {
+      _gScaleFactor = scaleFactor;
+      ExecutiveInvalidateRep(G, NULL, cRepLabel, cRepInvRep);
+      OrthoCommandIn(G, "viewport");
+    } else {
+      SettingSetGlobal_i(G, cSetting_display_scale_factor, 1);
+      PRINTFB(G, FB_Setting, FB_Warnings)
+        "Setting-Error: Cannot set a display scale factor of 0\n"
+        ENDFB(G);
+    }
+  }
   default:
     break;
   }
@@ -2882,7 +2909,7 @@ void SettingInitGlobal(PyMOLGlobals * G, int alloc, int reset_gui, int use_defau
     if(G->Option->stereo_mode) {
       set_i(I, cSetting_stereo_mode, G->Option->stereo_mode);
     } else if(G->StereoCapable || G->Option->blue_line) {
-      set_i(I, cSetting_stereo_mode, 1);      /* quadbuffer if we can */
+      set_i(I, cSetting_stereo_mode, cStereo_quadbuffer);      /* quadbuffer if we can */
     }
 
     /* In order to get electrostatic potentials in kT from the Coulomb equation... 
@@ -2954,9 +2981,10 @@ void SettingInitGlobal(PyMOLGlobals * G, int alloc, int reset_gui, int use_defau
 StateIterator::StateIterator(PyMOLGlobals * G, CSetting * set, int state_, int nstate) {
   if(state_ == -2) {
     // current state
-    state = SettingGet_i(G, set, NULL, cSetting_state) - 1;
-    end = state + 1;
-  } else if(state_ == -1) {
+    state_ = SettingGet_i(G, set, NULL, cSetting_state) - 1;
+  }
+
+  if(state_ == -1) {
     // all states
     state = 0;
     end = nstate;
