@@ -584,13 +584,37 @@ int AtomInfoKnownWaterResName(PyMOLGlobals * G, const char *resn)
 }
 
 int AtomInfoKnownPolymerResName(const char *resn)
+{
+  return AtomInfoKnownProteinResName(resn) ||
+         AtomInfoKnownNucleicResName(resn);
+}
+
+int AtomInfoKnownNucleicResName(const char *resn)
+{
+  if (resn[0] == 'D') {
+    // Deoxy ribonucleotide
+    ++resn;
+  }
+
+  switch (resn[0]) {
+    case 'A':
+    case 'C':
+    case 'G':
+    case 'I':
+    case 'T':
+    case 'U':
+      if (!resn[1])
+        return true;
+  }
+
+  return false;
+}
+
+int AtomInfoKnownProteinResName(const char *resn)
 {
   switch (resn[0]) {
   case 'A':
     switch (resn[1]) {
-    case 0: /*  A*/
-      return true;
-      break;
     case 'L':
       switch (resn[2]) {
       case 'A': /* ALA */
@@ -619,8 +643,6 @@ int AtomInfoKnownPolymerResName(const char *resn)
     break;
   case 'C':
     switch (resn[1]) {
-    case 0: /* C */
-      return true;
     case 'Y':
       switch (resn[2]) {
       case 'S': /* CYS */
@@ -631,51 +653,8 @@ int AtomInfoKnownPolymerResName(const char *resn)
       break;
     }
     break;
-  case 'D':
-    switch (resn[1]) {
-    case 'G': /* DG */
-      switch (resn[2]) {
-      case 0:
-        return true;
-        break;
-      }
-    case 'C': /* DC */
-      switch (resn[2]) {
-      case 0:
-        return true;
-        break;
-      }
-    case 'T': /* DT */
-      switch (resn[2]) {
-      case 0:
-        return true;
-        break;
-      }
-    case 'A': /* DA */
-      switch (resn[2]) {
-      case 0:
-        return true;
-        break;
-      }
-    case 'U': /* DU */
-      switch (resn[2]) {
-      case 0:
-        return true;
-        break;
-      }
-    case 'I': /* DI */
-      switch (resn[2]) {
-      case 0:
-        return true;
-        break;
-      }
-    }
-    break;
   case 'G':
     switch (resn[1]) {
-    case 0: /* G */
-      return true;
-      break;
     case 'L':
       switch (resn[2]) {
       case 'N': /* GLN */
@@ -703,6 +682,7 @@ int AtomInfoKnownPolymerResName(const char *resn)
       }
       break;
     }
+    break;
   case 'I':
     switch (resn[1]) {
     case 'L':
@@ -785,8 +765,6 @@ int AtomInfoKnownPolymerResName(const char *resn)
     break;
   case 'T':
     switch (resn[1]) {
-    case 0: /* T */
-      return true;
     case 'H':
       switch (resn[2]) {
       case 'R': /* THR */
@@ -810,13 +788,6 @@ int AtomInfoKnownPolymerResName(const char *resn)
       break;
     }
     break;
-  case 'U':
-    switch (resn[1]) {
-    case 0: /* U */
-      return true;
-      break;
-    }
-    break;
   case 'V':
     switch (resn[1]) {
     case 'A':
@@ -852,9 +823,6 @@ bool AtomResiFromResv(char *resi, size_t size, int resv, char inscode) {
 /*========================================================================*/
 PyObject *AtomInfoAsPyList(PyMOLGlobals * G, const AtomInfoType * I)
 {
-#ifdef _PYMOL_NOPY
-  return NULL;
-#else
   PyObject *result = NULL;
 
   result = PyList_New(48);
@@ -921,14 +889,10 @@ PyObject *AtomInfoAsPyList(PyMOLGlobals * G, const AtomInfoType * I)
   PyList_SetItem(result, 47, PyString_FromString(LexStr(G, I->custom)));
 
   return (PConvAutoNone(result));
-#endif
 }
 
 int AtomInfoFromPyList(PyMOLGlobals * G, AtomInfoType * I, PyObject * list)
 {
-#ifdef _PYMOL_NOPY
-  return 0;
-#else
   int ok = true;
   int tmp_int;
   ov_size ll = 0;
@@ -1059,7 +1023,6 @@ int AtomInfoFromPyList(PyMOLGlobals * G, AtomInfoType * I, PyObject * list)
     PCONVPYSTRTOLEXIDX(47, I->custom);
   }
   return (ok);
-#endif
 }
 
 void AtomInfoCopy(PyMOLGlobals * G, const AtomInfoType * src, AtomInfoType * dst, int copy_properties)
@@ -1289,7 +1252,6 @@ int AtomInfoUniquefyNames(PyMOLGlobals * G, const AtomInfoType * atInfo0, int n0
       } else {
         sprintf(name, "%1d%1s%02d", c / 100, ai1->elem, c % 100);
       }
-      name[4] = 0;              /* just is case we go over */
       LexAssign(G, ai1->name, name);
       result++;
       c = c + 1;
@@ -1913,6 +1875,12 @@ const ElementTableItemType ElementTable[] = {
   {"darmstadtium",      "Ds",   1.80, 281.000000},
   {"roentgenium",       "Rg",   1.80, 281.000000},
   {"copernicium",       "Cn",   1.80, 285.000000},
+  {"nihonium",          "Nh",   1.80, 286.000000},
+  {"flerovium",         "Fl",   1.80, 289.000000},
+  {"moscovium",         "Mc",   1.80, 290.000000},
+  {"livermorium",       "Lv",   1.80, 293.000000},
+  {"tennessine",        "Ts",   1.80, 294.000000},
+  {"oganesson",         "Og",   1.80, 294.000000},
   {NULL,                NULL,   0.00,   0.000000}
 };
 
@@ -2082,15 +2050,14 @@ static int AtomInfoCompare(PyMOLGlobals *G, const AtomInfoType *at1, const AtomI
 
   if (at1->priority != at2->priority) return (at1->priority < at2->priority) ? -1 : 1;
 
-  // empty alt code goes last: 'A' < 'B' < ''
+  // Changed (PyMOL 2.1): name before alt
+  if ((wc = AtomInfoNameCompare(G, at1->name, at2->name))) return wc;
+
+  // Changed (PyMOL 2.1): empty alt goes first: '' < 'A' < 'B'
   if (at1->alt[0] != at2->alt[0]) {
-    if (!at2->alt[0]) return -1;
-    if (!at1->alt[0]) return 1;
     return (at1->alt[0] < at2->alt[0]) ? -1 : 1;
   }
 
-  if ((wc = AtomInfoNameCompare(G, at1->name, at2->name))) return wc;
-
 rank_compare:
   if (!ignore_rank && at1->rank != at2->rank) return (at1->rank < at2->rank) ? -1 : 1;
 
@@ -2117,10 +2084,18 @@ int AtomInfoCompareIgnoreHet(PyMOLGlobals * G, const AtomInfoType * at1, const A
   return AtomInfoCompare(G, at1, at2, true, false);
 }
 
+/*
+ * Function only used for matching atoms of two aligned residues.
+ *
+ * Changed (PyMOL 2.1):
+ * Consider alt codes to be equal if at least one is empty.
+ * Otherwise, alt-code (C-alpha) atoms can't be aligned to non-alt atoms
+ * (also affects morphing).
+ */
 int AtomInfoNameOrder(PyMOLGlobals * G, const AtomInfoType * at1, const AtomInfoType * at2)
 {
   int result;
-  if(at1->alt[0] == at2->alt[0]) {
+  if(at1->alt[0] == at2->alt[0] || !at1->alt[0] || !at2->alt[0]) {
     if(at1->priority == at2->priority) {
       result = AtomInfoNameCompare(G, at1->name, at2->name);
     } else if(at1->priority < at2->priority) {
@@ -2128,7 +2103,7 @@ int AtomInfoNameOrder(PyMOLGlobals * G, const AtomInfoType * at1, const AtomInfo
     } else {
       result = 1;
     }
-  } else if((!at2->alt[0]) || (at1->alt[0] && ((at1->alt[0] < at2->alt[0])))) {
+  } else if(at1->alt[0] < at2->alt[0]) {
     result = -1;
   } else {
     result = 1;
@@ -2381,7 +2356,7 @@ static int get_protons(const char * symbol)
   }
 
   // check second letter for lower case
-  if (isupper(symbol[1])) {
+  if (isupper(symbol[1]) && strcmp(symbol, "LP") != 0) {
     UtilNCopy(titleized, symbol, 4);
     titleized[1] = tolower(symbol[1]);
     symbol = titleized;
@@ -2449,16 +2424,15 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
     n = LexStr(G, I->name);
     while(((*n) >= '0') && ((*n) <= '9') && (*(n + 1)))
       n++;
-    while((((*n) >= 'A') && ((*n) <= 'Z')) || (((*n) >= 'a') && ((*n) <= 'z'))) {
-      *(e++) = *(n++);
-    }
-    *e = 0;
-    e = I->elem;
+    strncpy(e, n, cElemNameLen);
     switch (*e) {
+    case '\0':
+      break;
     case 'C':
       if(*(e + 1) == 'A') {
         if(!WordMatchExact(G, G->lex_const.CA, I->resn, true)
            && (!WordMatchExact(G, "CA+", LexStr(G, I->resn), true)))
+          /* CA intpreted as carbon, not calcium */
           *(e + 1) = 0;
       } else if(!((*(e + 1) == 'a') ||  /* CA intpreted as carbon, not calcium */
                   (*(e + 1) == 'l') || (*(e + 1) == 'L') ||
@@ -2500,10 +2474,8 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
     case 'Q':
       *(e + 1) = 0;
       break;
-    default:
-      break;
     }
-    if(*(e + 1))
+    if(*(e + 1) && (e[1] != 'P' || e[0] != 'L'))
       *(e + 1) = tolower(*(e + 1));
   }
 
@@ -2836,3 +2808,169 @@ void atomicnumber2elem(char * dst, int protons) {
     strncpy(dst, ElementTable[protons].symbol, cElemNameLen);
 }
 
+/*
+ * Get a string representation of the stereo configuration. Either
+ * R/S chirality, if set, or the SDF odd/even parity, if set.
+ */
+const char * AtomInfoGetStereoAsStr(const AtomInfoType * ai) {
+  switch (ai->mmstereo) {
+    case 1 /* MMSTEREO_CHIRALITY_R */: return "R";
+    case 2 /* MMSTEREO_CHIRALITY_S */: return "S";
+  }
+
+  switch (ai->stereo) {
+    case SDF_CHIRALITY_ODD:  return "odd";
+    case SDF_CHIRALITY_EVEN: return "even";
+  }
+
+  if (ai->mmstereo || ai->stereo) {
+    return "?";
+  }
+
+  return "";
+}
+
+/*
+ * Set stereochemistry. Valid are: R, S, N[one], E[ven], O[dd]
+ */
+void AtomInfoSetStereo(AtomInfoType * ai, const char * stereo) {
+  switch (toupper(stereo[0])) {
+    case 'R':  ai->mmstereo = 1; ai->stereo = 0; break; // MMSTEREO_CHIRALITY_R
+    case 'S':  ai->mmstereo = 2; ai->stereo = 0; break; // MMSTEREO_CHIRALITY_S
+    case 'E':  ai->mmstereo = 0; ai->stereo = SDF_CHIRALITY_EVEN;       break;
+    case 'O':  ai->mmstereo = 0; ai->stereo = SDF_CHIRALITY_ODD;        break;
+    case 'N': case 0: ai->mmstereo = ai->stereo = 0;    break;
+    default:   ai->mmstereo = ai->stereo = 3;           break;
+  }
+}
+
+
+/*
+ * Get column aligned (left space padded) PDB residue name
+ *
+ * resn: output buffer
+ */
+void AtomInfoGetAlignedPDBResidueName(PyMOLGlobals * G,
+    const AtomInfoType * ai,
+    ResName & resn)
+{
+  sprintf(resn, "%3.4s", LexStr(G, ai->resn));
+  if(SettingGetGlobal_b(G, cSetting_pdb_truncate_residue_name)) {
+    resn[3] = 0;                /* enforce 3-letter residue name in PDB files */
+  }
+}
+
+
+/*
+ * Get column aligned (left space padded) PDB atom name
+ *
+ * resn: space padded residue name
+ * name: output buffer
+ */
+void AtomInfoGetAlignedPDBAtomName(PyMOLGlobals * G,
+    const AtomInfoType * ai,
+    const ResName & resn,
+    AtomName & name)
+{
+  int literal = SettingGetGlobal_b(G, cSetting_pdb_literal_names);
+  int reformat = SettingGetGlobal_i(G, cSetting_pdb_reformat_names_mode);
+
+  // default is "literal=0" and "reformat=0" (no reformatting)
+
+  const char * ai_name = LexStr(G, ai->name);
+  auto ai_name_len = strlen(ai_name);
+  bool start_column_1 = false;
+
+  UtilNCopy(name, ai_name, 5);
+
+  if(!ai->name) {
+    if(!ai->elem[1])
+      sprintf(name, " %s", ai->elem);
+    else
+      sprintf(name, "%s", ai->elem);
+  } else if(!literal) {
+    if(ai_name_len < 4) {  /* atom name less than length 4 */
+      if(!isdigit(name[0])) {     /* doesn't start with a number */
+        if((toupper(ai->elem[0]) == toupper(name[0])) && ((!ai->elem[1]) || /* symbol len = 1 */
+              (toupper(ai->elem[1]) == toupper(name[1])))) {        /* matched len 2 */
+          /* starts with corrent atomic symbol, so */
+          if(!ai->elem[1]) { /* symbol len = 1 */
+            switch (reformat) {
+            case 1:            /* pdb with internal pdb */
+            case 3:            /* pdb with internal iupac */
+              if((ai->elem[0] == 'H') && ai_name_len > 2) {
+                AtomInfoGetPDB3LetHydroName(G, resn, ai_name, name);
+                break;
+              }
+            default:           /* otherwise, start in column 1 */
+              start_column_1 = true;
+              break;
+            }
+          }
+        } else {                /* name doesn't start with atomic symbol */
+          /* then just place it in column 1 as usual */
+          start_column_1 = true;
+        }
+      } else {                  /* name starts with a number */
+        switch (reformat) {
+        case 2:                /* make Amber compliant */
+          if((ai->elem[0] == name[1]) &&
+             ((!ai->elem[1]) || (toupper(ai->elem[1]) == toupper(name[2])))) {
+            /* rotate the name to place atom symbol in column 0 to comply with Amber PDB format */
+            name[3] = name[0];
+            name[0] = ' ';
+          }
+          break;
+        }
+      }                         /* just stick it in column 0 and hope for the best */
+    } else {                    /* if name is length 4 */
+      if((ai->elem[0] == name[0]) && ((!ai->elem[1]) ||     /* symbol len = 1 */
+                                          (toupper(ai->elem[1]) == toupper(name[1])))) {    /* matched len 2 */
+        /* name starts with the atomic symbol */
+        if((!ai->elem[1]) && (ai->elem[0])) {   /* but if element is one letter... */
+          switch (reformat) {
+          case 1:              /* retaining PDB compliance throughout, or */
+          case 3:              /* saving as PDB compliant, but use IUPAC within PyMOL */
+            if(isdigit(name[3])) {  /* and last character is a number */
+              /* rotate the name to place atom symbol in column 1 to comply with PDB format */
+              name[0] = ai_name[3];
+              name[1] = ai_name[0];
+              name[2] = ai_name[1];
+              name[3] = ai_name[2];
+              name[4] = 0;
+            }
+            break;
+          }
+        }
+      } else {                  /* name does not start with the symbol... */
+        if(reformat == 2) {     /* AMBER compliance mode */
+          if(isdigit(name[0])) {
+            if((ai->elem[0] == name[1]) &&
+               ((!(ai->elem[1])) || (toupper(ai->elem[1]) == toupper(name[2])))) {
+              /* rotate the name to place atom symbol in column 0 to comply with Amber PDB format */
+              name[0] = ai_name[1];
+              name[1] = ai_name[2];
+              name[2] = ai_name[3];
+              name[3] = ai_name[0];
+              name[4] = 0;
+            }
+          }
+        }
+      }
+    }
+  } else {                      /* LITERAL mode: preserve what was in the original PDB as best PyMOL can
+                                   this should enable people to open and save amber pdb files without issues */
+    if (ai_name_len < 4 && !(ai->elem[1] && /* elem len = 2 */
+          toupper(ai->elem[0]) == toupper(name[0]) &&
+          toupper(ai->elem[1]) == toupper(name[1]))) {
+      start_column_1 = true;
+    }
+  }
+
+  if (start_column_1) {
+    name[0] = ' ';
+    UtilNCopy(name + 1, ai_name, 4);
+  }
+
+  name[4] = 0;
+}
