@@ -59,7 +59,9 @@ Z* -------------------------------------------------------------------
 #define cAtomFlag_study         0x00000020
 
 
-/* FLAGS 4-7 are reserved for additional molecular modeling tasks */
+/* FLAGS 6-7 are for polymer sub-classification */
+#define cAtomFlag_protein       0x00000040
+#define cAtomFlag_nucleic       0x00000080
 
 
 /* FLAGS 8-15 are free for end users to manipulate */
@@ -184,6 +186,10 @@ Z* -------------------------------------------------------------------
 
 #define cAN_U  92
 
+#define SDF_CHIRALITY_ODD    1          // odd  / clockwise
+#define SDF_CHIRALITY_EVEN   2          // even / counterclockwise
+#define SDF_CHIRALITY_EITHER 3          // either or unmarked
+
 typedef char Chain[2];
 typedef char SSType[2];
 typedef char SegIdent[cSegiLen + 1];
@@ -283,12 +289,23 @@ typedef struct AtomInfoType {
   unsigned char stereo : 2;     // 0-3 Only for SDF (MOL) format in/out
   unsigned char chemFlag : 2;   // 0,1,2
   unsigned char protekted : 2;  // 0,1,2
+  unsigned char mmstereo : 2;   // 0/R/S/?
 
   // methods
   bool isHydrogen() const {
     return protons == cAN_H;
   }
 
+  bool isMetal() const {
+    return (
+        (protons >  2 && protons <  5) ||
+        (protons > 10 && protons < 14) ||
+        (protons > 18 && protons < 32) ||
+        (protons > 36 && protons < 51) ||
+        (protons > 54 && protons < 85) ||
+        protons > 86);
+  }
+
   char getInscode(bool space=false) const {
     if (space && !inscode)
       return ' ';
@@ -407,6 +424,8 @@ inline bool AtomResiFromResv(char *resi, size_t size, const AtomInfoType * ai) {
 
 int AtomInfoKnownWaterResName(PyMOLGlobals * G, const char *resn);
 int AtomInfoKnownPolymerResName(const char *resn);
+int AtomInfoKnownProteinResName(const char *resn);
+int AtomInfoKnownNucleicResName(const char *resn);
 void AtomInfoGetPDB3LetHydroName(PyMOLGlobals * G, const char *resn, const char *iname, char *oname);
 
 #define cAIC_ct        0x0001
@@ -488,4 +507,13 @@ V AtomSettingGetWD(PyMOLGlobals * G, T * item, int index, V default_) {
 const char * AtomInfoGetStereoAsStr(const AtomInfoType * ai);
 void AtomInfoSetStereo(AtomInfoType * ai, const char * stereo);
 
+
+void AtomInfoGetAlignedPDBResidueName(PyMOLGlobals * G,
+    const AtomInfoType * ai,
+    ResName & resn);
+void AtomInfoGetAlignedPDBAtomName(PyMOLGlobals * G,
+    const AtomInfoType * ai,
+    const ResName & resn,
+    AtomName & name);
+
 #endif
