@@ -17,6 +17,7 @@
 #include "MemoryDebug.h"
 #include "Err.h"
 
+#include "AssemblyHelpers.h"
 #include "AtomInfo.h"
 #include "Base.h"
 #include "Executive.h"
@@ -30,6 +31,7 @@
 #include "Util2.h"
 #include "Vector.h"
 #include "Lex.h"
+#include "strcasecmp.h"
 
 // canonical amino acid three letter codes
 const char * aa_three_letter[] = {
@@ -374,6 +376,21 @@ static bond_dict_t * get_global_components_bond_dict(PyMOLGlobals * G) {
   return &bond_dict;
 }
 
+/*
+ * True for N-H1 and N-H3, those are not in the chemical components dictionary.
+ */
+static bool is_N_H1_or_H3(PyMOLGlobals * G,
+    const AtomInfoType * a1,
+    const AtomInfoType * a2) {
+  if (a2->name == G->lex_const.N) {
+    a2 = a1;
+  } else if (a1->name != G->lex_const.N) {
+    return false;
+  }
+
+  return (a2->name == G->lex_const.H1 || a2->name == G->lex_const.H3);
+}
+
 /*
  * Add bonds for one residue, with atoms spanning from i_start to i_end-1,
  * based on components.cif
@@ -415,7 +432,10 @@ static void ConnectComponent(ObjectMolecule * I, int i_start, int i_end,
       // lookup if atoms are bonded
       order = res_dict->get(LexStr(G, a1->name), LexStr(G, a2->name));
       if (order < 0) {
-        continue;
+        if (!is_N_H1_or_H3(G, a1, a2) || GetDistance(I, i1, i2) > 1.2)
+          continue;
+
+        order = 1;
       }
 
       // make bond
@@ -621,35 +641,6 @@ static bool get_assembly_chains(PyMOLGlobals * G,
   return !assembly_chains.empty();
 }
 
-/*
- * Create a coordset for a chain (actually segi = label_asym_id) selection
- */
-static CoordSet *CoordSetCopyFilterChains(const CoordSet * other,
-    const AtomInfoType * atInfo,
-    const std::set<lexidx_t> &chains_set) {
-
-  std::vector<int> idxmap;
-  idxmap.reserve(other->NIndex);
-
-  for (int idx = 0; idx < other->NIndex; ++idx)
-    if (chains_set.count(atInfo[other->IdxToAtm[idx]].segi) > 0)
-      idxmap.push_back(idx);
-
-  CoordSet * cset = CoordSetNew(other->State.G);
-
-  cset->NIndex = idxmap.size();
-  cset->Coord = VLAlloc(float, cset->NIndex * 3);
-  cset->IdxToAtm = VLAlloc(int, cset->NIndex);
-  cset->Obj = other->Obj;
-
-  for (int idx = 0; idx < cset->NIndex; ++idx) {
-    cset->IdxToAtm[idx] = other->IdxToAtm[idxmap[idx]];
-    copy3f(other->coordPtr(idxmap[idx]), cset->coordPtr(idx));
-  }
-
-  return cset;
-}
-
 /*
  * Read assembly
  *
@@ -871,15 +862,31 @@ static CoordSet ** read_chem_comp_atom_model(PyMOLGlobals * G, cif_data * data,
 
   const cif_array *arr_x, *arr_y = NULL, *arr_z = NULL;
 
-  if ((arr_x = data->get_arr("_chem_comp_atom.pdbx_model_cartn_x_ideal"))) {
+  // setting to exclude one or more coordinate columns
+  unsigned mask = SettingGetGlobal_i(G, cSetting_chem_comp_cartn_use);
+  const char * feedback = "";
+
+  if (!mask) {
+    mask = 0xFF;
+  }
+
+  if ((mask & 0x01)
+      && (arr_x = data->get_arr("_chem_comp_atom.pdbx_model_cartn_x_ideal"))
+      && !arr_x->is_missing_all()) {
     arr_y = data->get_arr("_chem_comp_atom.pdbx_model_cartn_y_ideal");
     arr_z = data->get_arr("_chem_comp_atom.pdbx_model_cartn_z_ideal");
-  } else if ((arr_x = data->get_arr("_chem_comp_atom.x"))) {
-    arr_y = data->get_arr("_chem_comp_atom.y");
-    arr_z = data->get_arr("_chem_comp_atom.z");
-  } else if ((arr_x = data->get_arr("_chem_comp_atom.model_cartn_x"))) {
+    feedback = ".pdbx_model_Cartn_{x,y,z}_ideal";
+  } else if ((mask & 0x02)
+      && (arr_x = data->get_arr("_chem_comp_atom.model_cartn_x"))) {
     arr_y = data->get_arr("_chem_comp_atom.model_cartn_y");
     arr_z = data->get_arr("_chem_comp_atom.model_cartn_z");
+    feedback = ".model_Cartn_{x,y,z}";
+  } else if ((mask & 0x04)
+      && (arr_x = data->get_arr("_chem_comp_atom.x"))
+      && !arr_x->is_missing_all()) {
+    arr_y = data->get_arr("_chem_comp_atom.y");
+    arr_z = data->get_arr("_chem_comp_atom.z");
+    feedback = ".{x,y,z}";
   }
 
   if (!arr_x || !arr_y || !arr_z) {
@@ -887,7 +894,8 @@ static CoordSet ** read_chem_comp_atom_model(PyMOLGlobals * G, cif_data * data,
   }
 
   PRINTFB(G, FB_Executive, FB_Details)
-    " ExecutiveLoad-Detail: Detected chem_comp CIF\n" ENDFB(G);
+    " ExecutiveLoad-Detail: Detected chem_comp CIF (%s)\n", feedback
+    ENDFB(G);
 
   const cif_array * arr_name            = data->get_opt("_chem_comp_atom.atom_id");
   const cif_array * arr_symbol          = data->get_opt("_chem_comp_atom.type_symbol");
@@ -923,7 +931,7 @@ static CoordSet ** read_chem_comp_atom_model(PyMOLGlobals * G, cif_data * data,
     ai->hetatm = 1;
 
     ai->visRep = auto_show;
-    ai->stereo = convertCharToStereo(arr_stereo->as_s(i)[0]);
+    AtomInfoSetStereo(ai, arr_stereo->as_s(i));
 
     AtomInfoAssignParameters(G, ai);
     AtomInfoAssignColors(G, ai);
@@ -1042,6 +1050,7 @@ static CoordSet ** read_atom_site(PyMOLGlobals * G, cif_data * data,
   const cif_array * arr_color = data->get_arr("_atom_site.pymol_color");
   const cif_array * arr_reps  = data->get_arr("_atom_site.pymol_reps");
   const cif_array * arr_ss    = data->get_opt("_atom_site.pymol_ss");
+  const cif_array * arr_formal_charge = data->get_opt("_atom_site.pdbx_formal_charge");
 
   if (!arr_chain)
     arr_chain = arr_segi;
@@ -1164,6 +1173,7 @@ static CoordSet ** read_atom_site(PyMOLGlobals * G, cif_data * data,
     }
 
     ai->ssType[0] = arr_ss->as_s(i)[0];
+    ai->formalCharge = arr_formal_charge->as_i(i);
 
     AtomInfoAssignParameters(G, ai);
 
@@ -1986,16 +1996,71 @@ static BondType * read_chem_comp_bond(PyMOLGlobals * G, cif_data * data,
   return bondvla;
 }
 
+/*
+ * Read bonds from _pymol_bond (non-standard extension)
+ *
+ * return: BondType VLA
+ */
+static BondType * read_pymol_bond(PyMOLGlobals * G, cif_data * data,
+    AtomInfoType * atInfo) {
+
+  const cif_array *col_ID_1, *col_ID_2, *col_order;
+
+  if ((col_ID_1    = data->get_arr("_pymol_bond.atom_site_id_1")) == NULL ||
+      (col_ID_2    = data->get_arr("_pymol_bond.atom_site_id_2")) == NULL ||
+      (col_order   = data->get_arr("_pymol_bond.order")) == NULL)
+    return NULL;
+
+  int nrows = col_ID_1->get_nrows();
+  int nAtom = VLAGetSize(atInfo);
+
+  BondType *bondvla, *bond;
+  bondvla = bond = VLACalloc(BondType, nrows);
+
+  // ID -> atom index
+  std::map<int, int> id_dict;
+
+  for (int atm = 0; atm < nAtom; ++atm) {
+    id_dict[atInfo[atm].id] = atm;
+  }
+
+  for (int i = 0; i < nrows; i++) {
+    auto key1 = col_ID_1->as_i(i);
+    auto key2 = col_ID_2->as_i(i);
+    auto order_value = col_order->as_i(i);
+
+    int i1, i2;
+    if (find2(id_dict, i1, key1, i2, key2)) {
+      BondTypeInit2(bond++, i1, i2, order_value);
+    } else {
+      PRINTFB(G, FB_Executive, FB_Details)
+        " Executive-Detail: _pymol_bond name lookup failed: %d %d\n",
+        key1, key2 ENDFB(G);
+    }
+  }
+
+  return bondvla;
+}
+
 /*
  * Create a new (multi-state) object-molecule from datablock
  */
-static ObjectMolecule *ObjectMoleculeReadCifData(PyMOLGlobals * G, cif_data * datablock, int discrete)
+static ObjectMolecule *ObjectMoleculeReadCifData(PyMOLGlobals * G,
+    cif_data * datablock, int discrete, bool quiet)
 {
   CoordSet ** csets = NULL;
   int ncsets;
   CifContentInfo info(G, SettingGetGlobal_b(G, cSetting_cif_use_auth));
   const char * assembly_id = SettingGetGlobal_s(G, cSetting_assembly);
 
+  // title "echo tag"
+  const char * title = datablock->get_opt("_struct.title")->as_s();
+  if (!quiet && title[0] &&
+      strstr(SettingGetGlobal_s(G, cSetting_pdb_echo_tags), "TITLE")) {
+    PRINTFB(G, FB_ObjectMolecule, FB_Details)
+      "TITLE     %s\n", title ENDFB(G);
+  }
+
   if (assembly_id && assembly_id[0]) {
     if (!get_assembly_chains(G, datablock, info.chains_filter, assembly_id))
       PRINTFB(G, FB_Executive, FB_Details)
@@ -2096,7 +2161,9 @@ static ObjectMolecule *ObjectMoleculeReadCifData(PyMOLGlobals * G, cif_data * da
 
       break;
     case CIF_MMCIF:
-      if (cset) {
+      I->Bond = read_pymol_bond(G, datablock, I->AtomInfo);
+
+      if (cset && !I->Bond) {
         // sort atoms internally
         ObjectMoleculeSort(I);
 
@@ -2154,23 +2221,7 @@ static ObjectMolecule *ObjectMoleculeReadCifData(PyMOLGlobals * G, cif_data * da
     CoordSet **assembly_csets = read_pdbx_struct_assembly(G, datablock,
         I->AtomInfo, cset, assembly_id);
 
-    if (assembly_csets) {
-      // remove asymetric unit coordinate sets
-      for (int i = 0; i < I->NCSet; ++i)
-        if (I->CSet[i])
-          I->CSet[i]->fFree();
-      VLAFreeP(I->CSet);
-
-      // get assembly coordinate sets into ObjectMolecule
-      I->CSet = assembly_csets;
-      I->NCSet = VLAGetSize(assembly_csets);
-      I->updateAtmToIdx();
-
-      // all_states for multi-model assembly
-      if (I->NCSet > 1) {
-        SettingSet(cSetting_all_states, true, &I->Obj);
-      }
-    }
+    ObjectMoleculeSetAssemblyCSets(I, assembly_csets);
   }
 
   // computationally intense update tasks
@@ -2221,11 +2272,11 @@ ObjectMolecule *ObjectMoleculeReadCifStr(PyMOLGlobals * G, ObjectMolecule * I,
 #endif
 
   for (auto it = cif->datablocks.begin(); it != cif->datablocks.end(); ++it) {
-    ObjectMolecule * obj = ObjectMoleculeReadCifData(G, it->second, discrete);
+    ObjectMolecule * obj = ObjectMoleculeReadCifData(G, it->second, discrete, quiet);
 
     if (!obj) {
-      PRINTFB(G, FB_ObjectMolecule, FB_Errors)
-        " mmCIF-Error: no coordinates found in data_%s\n", it->first ENDFB(G);
+      PRINTFB(G, FB_ObjectMolecule, FB_Warnings)
+        " mmCIF-Warning: no coordinates found in data_%s\n", it->first ENDFB(G);
       continue;
     }
 
@@ -2242,7 +2293,7 @@ ObjectMolecule *ObjectMoleculeReadCifStr(PyMOLGlobals * G, ObjectMolecule * I,
 
     // multiplexing
     ObjectSetName((CObject*) obj, it->first);
-    ExecutiveDelete(G, it->first);
+    ExecutiveDelete(G, obj->Obj.Name);
     ExecutiveManageObject(G, (CObject*) obj, zoom, true);
   }
 
