@@ -461,12 +461,6 @@ static CSetting **ObjectDistGetSettingHandle(ObjectDist * I, int state)
 {
   if(state < 0) {
     return (&I->Obj.Setting);
-  } else if(state < I->NDSet) {
-    if(I->DSet[state]) {
-      return (&I->DSet[state]->Setting);
-    } else {
-      return (NULL);
-    }
   } else {
     return (NULL);
   }
@@ -495,8 +489,10 @@ ObjectDist *ObjectDistNew(PyMOLGlobals * G)
   I->Obj.fUpdate = (void (*)(CObject *)) ObjectDistUpdate;
   I->Obj.fInvalidate = (void (*)(CObject *, int, int, int)) ObjectDistInvalidate;
   I->Obj.fGetNFrame = (int (*)(CObject *)) ObjectDistGetNFrames;
+#if 0
   I->Obj.fGetSettingHandle = (CSetting ** (*)(CObject *, int state))
     ObjectDistGetSettingHandle;
+#endif
   I->Obj.fDescribeElement = NULL;
   I->Obj.Color = ColorGetIndex(G, "dash");
   return (I);
