@@ -278,25 +278,20 @@ PyObject *ObjectMeshAsPyList(ObjectMesh * I)
       retObjectCGO = ObjectCGOFromCGO(I->Obj.G, retObjectCGO, cgo, a);
       
     }
-    ObjectSetRepVis(&retObjectCGO->Obj, cRepMesh, 0);
-    ObjectSetRepVis(&retObjectCGO->Obj, cRepCGO, 1);
+    ObjectSetRepVisMask(&retObjectCGO->Obj, cRepCGOBit, cVis_AS);
     result = ObjectCGOAsPyList(retObjectCGO);
     ObjectCGOFree(retObjectCGO);
   }
   return (PConvAutoNone(result));
 }
 
-static void ObjectMeshStateFree(ObjectMeshState * ms)
+static void ObjectMeshStatePurge(ObjectMeshState * ms)
 {
   ObjectStatePurge(&ms->State);
   if(ms->Field) {
     IsosurfFieldFree(ms->State.G, ms->Field);
     ms->Field = NULL;
   }
-  VLAFreeP(ms->N);
-  VLAFreeP(ms->V);
-  FreeP(ms->VC);
-  FreeP(ms->RC);
   VLAFreeP(ms->AtomVertex);
   CGOFree(ms->shaderCGO);
   CGOFree(ms->shaderUnitCellCGO);
@@ -304,6 +299,15 @@ static void ObjectMeshStateFree(ObjectMeshState * ms)
   ms->Active = false;
 }
 
+static void ObjectMeshStateFree(ObjectMeshState * ms)
+{
+  ObjectMeshStatePurge(ms);
+  VLAFreeP(ms->N);
+  VLAFreeP(ms->V);
+  FreeP(ms->VC);
+  FreeP(ms->RC);
+}
+
 static void ObjectMeshFree(ObjectMesh * I)
 {
   int a;
@@ -1133,8 +1137,7 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
   if (!ok){
     I->Obj.fInvalidate(&I->Obj, cRepMesh, cRepInvPurge, -1);
     I->Obj.fInvalidate(&I->Obj, cRepCGO, cRepInvPurge, -1);
-    ObjectSetRepVis(&I->Obj, cRepMesh, 0);
-    ObjectSetRepVis(&I->Obj, cRepCGO, 0);
+    ObjectSetRepVisMask(&I->Obj, 0, cVis_AS);
   }
 
   return NULL;
@@ -1185,11 +1188,7 @@ ObjectMesh *ObjectMeshNew(PyMOLGlobals * G)
 void ObjectMeshStateInit(PyMOLGlobals * G, ObjectMeshState * ms)
 {
   if(ms->Active)
-    ObjectStatePurge(&ms->State);
-  if(ms->Field) {
-    IsosurfFieldFree(ms->State.G, ms->Field);
-    ms->Field = NULL;
-  }
+    ObjectMeshStatePurge(ms);
   ObjectStateInit(G, &ms->State);
   if(!ms->V) {
     ms->V = VLAlloc(float, 10000);
@@ -1197,9 +1196,6 @@ void ObjectMeshStateInit(PyMOLGlobals * G, ObjectMeshState * ms)
   if(!ms->N) {
     ms->N = VLAlloc(int, 10000);
   }
-  if(ms->AtomVertex) {
-    VLAFreeP(ms->AtomVertex);
-  }
   ms->N[0] = 0;
   ms->Active = true;
   ms->ResurfaceFlag = true;
