@@ -97,7 +97,7 @@ typedef struct ObjectMolecule {
   bool need_hetatm_classification;
 
   // methods
-  int getState();
+  int getState() const;
   bool setNDiscrete(int natom);
   bool updateAtmToIdx();
 } ObjectMolecule;
@@ -141,6 +141,7 @@ enum {
   PDB_VARIANT_DEFAULT = 0,
   PDB_VARIANT_PQR,
   PDB_VARIANT_PDBQT,
+  PDB_VARIANT_VDB,      /* VIPERdb */
 };
 
 typedef struct {
@@ -422,7 +423,7 @@ int ObjectMoleculeAutoDisableAtomNameWildcard(ObjectMolecule * I);
 
 void ObjectMoleculeSaveUndo(ObjectMolecule * I, int state, int log);
 void ObjectMoleculeUndo(ObjectMolecule * I, int dir);
-int ObjectMoleculePrepareAtom(ObjectMolecule * I, int index, AtomInfoType * ai);
+int ObjectMoleculePrepareAtom(ObjectMolecule * I, int index, AtomInfoType * ai, bool uniquefy=true);
 void ObjectMoleculeReplaceAtom(ObjectMolecule * I, int index, AtomInfoType * ai);
 int ObjectMoleculePreposReplAtom(ObjectMolecule * I, int index, AtomInfoType * ai);
 void ObjectMoleculeCreateSpheroid(ObjectMolecule * I, int average);
@@ -506,9 +507,15 @@ struct CoordSet *ObjectMoleculePDBStr2CoordSet(PyMOLGlobals * G,
                                                PDBInfoRec * pdb_info,
                                                int quiet, int *model_number);
 
+#ifndef NO_MMLIBS
 int ObjectMoleculeUpdateAtomTypeInfoForState(PyMOLGlobals * G, ObjectMolecule * obj, int state, int initialize, int format);
+#endif
+
+#ifndef _PYMOL_NO_UNDO
 void ObjectMoleculeSetAtomBondInfoTypeOldId(PyMOLGlobals * G, ObjectMolecule * obj);
 void ObjectMoleculeSetAtomBondInfoTypeOldIdToNegOne(PyMOLGlobals * G, ObjectMolecule * obj);
+#endif
+
 void ObjectMoleculeAdjustDiscreteAtmIdx(ObjectMolecule *I, int *lookup, int nAtom);
 
 void AtomInfoSettingGenerateSideEffects(PyMOLGlobals * G, ObjectMolecule *obj, int index, int id);
@@ -516,6 +523,8 @@ void AtomInfoSettingGenerateSideEffects(PyMOLGlobals * G, ObjectMolecule *obj, i
 int *AtomInfoGetSortedIndex(PyMOLGlobals * G, ObjectMolecule * obj, AtomInfoType * rec, int n,
                             int **outdex);
 
+ObjectMolecule *ObjectMoleculeReadMmtfStr(PyMOLGlobals * G, ObjectMolecule * I,
+    const char *st, int st_len, int frame, int discrete, int quiet, int multiplex, int zoom);
 ObjectMolecule *ObjectMoleculeReadCifStr(PyMOLGlobals * G, ObjectMolecule * I,
     const char *st, int frame, int discrete, int quiet, int multiplex, int zoom);
 
