@@ -8,6 +8,7 @@
 #include <map>
 #include <algorithm>
 #include <cstdarg>
+#include <clocale>
 
 #include "os_std.h"
 
@@ -113,19 +114,19 @@ struct AtomRef {
  * Abstract base class for exporting molecular selections
  */
 struct MoleculeExporter {
-  char* m_buffer = NULL; // out
-  int m_offset = 0;
+  char* m_buffer; // out
+  int m_offset;
 
 protected:
-  CoordSet        * m_last_cs  = NULL;
-  ObjectMolecule  * m_last_obj = NULL;
-  int m_last_state = -1;
+  CoordSet        * m_last_cs;
+  ObjectMolecule  * m_last_obj;
+  int m_last_state;
 
-  PyMOLGlobals * G = NULL;
+  PyMOLGlobals * G;
   SeleCoordIterator m_iter;
 
-  bool m_retain_ids = false;
-  int m_id = 0;
+  bool m_retain_ids;
+  int m_id;
 
   struct matrix_t {
     double storage[16];
@@ -154,6 +155,14 @@ public:
     m_buffer = VLAlloc(char, 1280);
     m_buffer[0] = '\0';
 
+    m_mat_ref.ptr = NULL;
+    m_offset = 0;
+    m_last_cs  = NULL;
+    m_last_obj = NULL;
+    m_last_state = -1;
+    m_retain_ids = false;
+    m_id = 0;
+
     setMulti(getMultiDefault());
   }
 
@@ -205,6 +214,8 @@ protected:
    * Get the current state title if not empty, or the object name otherwise.
    */
   const char * getTitleOrName() {
+    if (!m_iter.cs)
+      return "untitled";
     return m_iter.cs->Name[0] ? m_iter.cs->Name : m_iter.obj->Obj.Name;
   }
 
@@ -334,6 +345,9 @@ void MoleculeExporter::execute(int sele, int state) {
     endCoordSet();
   if (m_last_obj)
     endObject();
+  else if (m_multi == cMolExportGlobal)
+    // empty selection
+    beginMolecule();
 
   if (m_multi == cMolExportGlobal) {
     writeBonds();
@@ -403,9 +417,9 @@ void MoleculeExporter::populateBondRefs() {
 // ---------------------------------------------------------------------------------- //
 
 struct MoleculeExporterPDB : public MoleculeExporter {
-  bool m_conect_all = false;
-  bool m_conect_nodup = false;
-  bool m_mdl_written = false;
+  bool m_conect_all;
+  bool m_conect_nodup;
+  bool m_mdl_written;
   PDBInfoRec m_pdb_info;
 
   // quasi constructor
@@ -414,6 +428,8 @@ struct MoleculeExporterPDB : public MoleculeExporter {
 
     UtilZeroMem((void *) &m_pdb_info, sizeof(PDBInfoRec));
 
+    m_conect_all    = false;
+    m_mdl_written   = false;
     m_conect_nodup  = SettingGetGlobal_b(G, cSetting_pdb_conect_nodup);
     m_retain_ids    = SettingGetGlobal_b(G, cSetting_pdb_retain_ids);
   }
@@ -447,7 +463,8 @@ struct MoleculeExporterPDB : public MoleculeExporter {
 
     std::map<int, std::vector<int>> conect;
 
-    for (auto& bond : m_bonds) {
+    for (auto bond_it = m_bonds.begin(); bond_it != m_bonds.end(); ++bond_it) {
+      auto& bond = *bond_it;
       int order = m_conect_nodup ? 1 : bond.ref->order;
       for (int i = 0; i < 2; ++i) {
         for (int d = 0; d < order; ++d) {
@@ -459,7 +476,8 @@ struct MoleculeExporterPDB : public MoleculeExporter {
 
     m_bonds.clear();
 
-    for (auto& rec : conect) {
+    for (auto rec_it = conect.begin(); rec_it != conect.end(); ++rec_it) {
+      const auto& rec = *rec_it;
       for (int i = 0, i_end = rec.second.size(); i != i_end;) {
         m_offset += VLAprintf(m_buffer, m_offset, "CONECT%5d", rec.first);
         // up to 4 bonds per record
@@ -670,17 +688,74 @@ struct MoleculeExporterCIF : public MoleculeExporter {
 
   void writeBonds() {
     // TODO
+    // Bond categories:
+    // 1) within residue -> _chem_comp_bond
+    // 2) polymer links -> implicit
+    // 3) others -> _struct_conn
     m_bonds.clear();
   }
 
   bool isExcludedBond(int atm1, int atm2) {
     // TODO
+    // polymer links
     return true;
   }
 };
 
 // ---------------------------------------------------------------------------------- //
 
+/*
+ * Experimental PyMOL style annotated mmCIF. Exports colors, representation,
+ * and secondary structure.
+ */
+struct MoleculeExporterPMCIF : public MoleculeExporterCIF {
+  void beginMolecule() {
+    MoleculeExporterCIF::beginMolecule();
+
+    m_offset += VLAprintf(m_buffer, m_offset, "#\n"
+        "_atom_site.pymol_color\n"
+        "_atom_site.pymol_reps\n"
+        "_atom_site.pymol_ss\n");
+  }
+
+  void writeAtom() {
+    MoleculeExporterCIF::writeAtom();
+
+    const AtomInfoType * ai = m_iter.getAtomInfo();
+
+    m_offset += VLAprintf(m_buffer, m_offset,
+        "%d %d %s\n",
+        ai->color,
+        ai->visRep,
+        cifrepr(ai->ssType));
+  }
+
+  void writeBonds() {
+    if (m_bonds.empty())
+      return;
+
+    m_offset += VLAprintf(m_buffer, m_offset, "#\n"
+        "loop_\n"
+        "_pymol_bond.atom_site_id_1\n"
+        "_pymol_bond.atom_site_id_2\n"
+        "_pymol_bond.order\n");
+
+    for (auto bond_it = m_bonds.begin(); bond_it != m_bonds.end(); ++bond_it) {
+      const auto& bond = *bond_it;
+      m_offset += VLAprintf(m_buffer, m_offset, "%d %d %d\n",
+          bond.id1, bond.id2, bond.ref->order);
+    }
+
+    m_bonds.clear();
+  }
+
+  bool isExcludedBond(int atm1, int atm2) {
+    return false;
+  }
+};
+
+// ---------------------------------------------------------------------------------- //
+
 struct MoleculeExporterMOL : public MoleculeExporter {
   int m_chiral_flag;
   std::vector<AtomRef> m_atoms;
@@ -709,7 +784,8 @@ struct MoleculeExporterMOL : public MoleculeExporter {
         m_atoms.size(), m_bonds.size(), m_chiral_flag);
 
     // write atoms
-    for (auto& atom : m_atoms) {
+    for (auto atom_it = m_atoms.begin(); atom_it != m_atoms.end(); ++atom_it) {
+      const auto& atom = *atom_it;
       auto ai = atom.ref;
 
       m_offset += VLAprintf(m_buffer, m_offset, "M  V30 %d %s %.4f %.4f %.4f 0",
@@ -732,7 +808,8 @@ struct MoleculeExporterMOL : public MoleculeExporter {
 
     // write bonds
     int n_bonds = 0;
-    for (auto& bond : m_bonds) {
+    for (auto bond_it = m_bonds.begin(); bond_it != m_bonds.end(); ++bond_it) {
+      const auto& bond = *bond_it;
       m_offset += VLAprintf(m_buffer, m_offset, "M  V30 %d %d %d %d\n",
           ++n_bonds, bond.ref->order, bond.id1, bond.id2);
     }
@@ -753,7 +830,8 @@ struct MoleculeExporterMOL : public MoleculeExporter {
         (int) m_atoms.size(), (int) m_bonds.size(), m_chiral_flag);
 
     // write atoms
-    for (auto& atom : m_atoms) {
+    for (auto atom_it = m_atoms.begin(); atom_it != m_atoms.end(); ++atom_it) {
+      const auto& atom = *atom_it;
       auto ai = atom.ref;
       int chg = ai->formalCharge;
       m_offset += VLAprintf(m_buffer, m_offset,
@@ -765,7 +843,8 @@ struct MoleculeExporterMOL : public MoleculeExporter {
     m_atoms.clear();
 
     // write bonds
-    for (auto& bond : m_bonds) {
+    for (auto bond_it = m_bonds.begin(); bond_it != m_bonds.end(); ++bond_it) {
+      const auto& bond = *bond_it;
       m_offset += VLAprintf(m_buffer, m_offset, "%3d%3d%3d%3d  0  0  0\n",
           bond.id1, bond.id2, bond.ref->order, (int) bond.ref->stereo);
     }
@@ -790,8 +869,8 @@ struct MoleculeExporterMOL : public MoleculeExporter {
     MoleculeExporter::beginMolecule();
 
     m_offset += VLAprintf(m_buffer, m_offset,
-        "%s\n  PyMOL%03d          3D                             0\n\n",
-        getTitleOrName(), (_PyMOL_VERSION_int / 10) % 1000);
+        "%s\n  PyMOL%3.3s          3D                             0\n\n",
+        getTitleOrName(), _PyMOL_VERSION);
 
     m_chiral_flag = 0;
   }
@@ -907,7 +986,8 @@ struct MoleculeExporterMOL2 : public MoleculeExporter {
     m_offset += VLAprintf(m_buffer, m_offset, "@<TRIPOS>BOND\n");
 
     int bond_id = 0;
-    for (auto& bond : m_bonds) {
+    for (auto bond_it = m_bonds.begin(); bond_it != m_bonds.end(); ++bond_it) {
+      const auto& bond = *bond_it;
       m_offset += VLAprintf(m_buffer, m_offset, "%d %d %d %s\n",
           ++bond_id,
           bond.id1,
@@ -924,7 +1004,8 @@ struct MoleculeExporterMOL2 : public MoleculeExporter {
     m_offset += VLAprintf(m_buffer, m_offset, "@<TRIPOS>SUBSTRUCTURE\n");
 
     int subst_id = 0;
-    for (auto& subst : m_substs) {
+    for (auto subst_it = m_substs.begin(); subst_it != m_substs.end(); ++subst_it) {
+      const auto& subst = *subst_it;
       const auto& ai = subst.ai;
       m_offset += VLAprintf(m_buffer, m_offset, "%d\t%s%d%.1s\t%d\t%s\t1 %s\t%s\n",
           ++subst_id,
@@ -945,7 +1026,14 @@ struct MoleculeExporterMOL2 : public MoleculeExporter {
 struct MoleculeExporterMAE : public MoleculeExporter {
   int m_n_atoms;
   int m_n_atoms_offset;
-  int m_n_arom_bonds = 0;
+  int m_n_arom_bonds;
+
+  // quasi constructor
+  void init(PyMOLGlobals * G_) {
+    MoleculeExporter::init(G_);
+
+    m_n_arom_bonds = 0;
+  }
 
   int getMultiDefault() const {
     // multi-entry format
@@ -1008,16 +1096,23 @@ struct MoleculeExporterMAE : public MoleculeExporter {
       strcpy(inscode, "<>");
     }
 
+    ResName resn = "";
+    AtomName name = "X";
+    if (ai->resn)
+      AtomInfoGetAlignedPDBResidueName(G, ai, resn);
+    if (ai->name)
+      AtomInfoGetAlignedPDBAtomName(G, ai, resn, name);
+
     m_offset += VLAprintf(m_buffer, m_offset,
-        "%d %d %.3f %.3f %.3f %d %s %s %s %s %d %d %02X%02X%02X %d %.2f %d\n", // %d %d\n",
+        "%d %d %.3f %.3f %.3f %d %s %s \"%-4s\" \"%-4s\" %d %d %02X%02X%02X %d %.2f %d\n",
         getTmpID(),
         getMacroModelAtomType(ai),
         m_coord[0], m_coord[1], m_coord[2],
         ai->resv,
         inscode,
         ai->chain ? LexStr(G, ai->chain) : "\" \"",
-        ai->resn  ? LexStr(G, ai->resn)  : "\"\"",
-        ai->name  ? LexStr(G, ai->name)  : "X",
+        resn,
+        name,
         ai->protons,
         ai->formalCharge,
         int(rgb[0] * 255),
@@ -1049,7 +1144,8 @@ struct MoleculeExporterMAE : public MoleculeExporter {
           ":::\n", (int) m_bonds.size());
 
       int b = 0;
-      for (auto& bond : m_bonds) {
+      for (auto bond_it = m_bonds.begin(); bond_it != m_bonds.end(); ++bond_it) {
+        const auto& bond = *bond_it;
         int order = bond.ref->order;
         if (order > 3) {
           ++m_n_arom_bonds;
@@ -1171,6 +1267,8 @@ unique_vla_ptr<char> MoleculeExporterGetStr(PyMOLGlobals * G,
 
   if (strcmp(format, "pdb") == 0) {
     exporter = new MoleculeExporterPDB;
+  } else if (strcmp(format, "pmcif") == 0) {
+    exporter = new MoleculeExporterPMCIF;
   } else if (strcmp(format, "cif") == 0) {
     exporter = new MoleculeExporterCIF;
   } else if (strcmp(format, "sdf") == 0) {
@@ -1191,6 +1289,10 @@ unique_vla_ptr<char> MoleculeExporterGetStr(PyMOLGlobals * G,
     return NULL;
   }
 
+  // Ensure "." decimal point in printf. It's possible to change this from
+  // Python, so don't rely on a persistent global value.
+  std::setlocale(LC_NUMERIC, "C");
+
   exporter->init(G);
   exporter->setMulti(multi);
   exporter->setRefObject(ref_object, ref_state);
@@ -1207,6 +1309,68 @@ unique_vla_ptr<char> MoleculeExporterGetStr(PyMOLGlobals * G,
 /*========================================================================*/
 
 #ifndef _PYMOL_NOPY
+/*
+ * See MoleculeExporterGetPyBonds
+ */
+struct MoleculeExporterPyBonds : public MoleculeExporter {
+  PyObject *m_bond_list; // out
+
+protected:
+  int getMultiDefault() const {
+    // single-entry format
+    return cMolExportGlobal;
+  }
+
+  void writeAtom() {}
+
+  void writeBonds() {
+    size_t nBond = m_bonds.size();
+    m_bond_list = PyList_New(nBond);
+
+    for (size_t b = 0; b < nBond; ++b) {
+      const auto& bond = m_bonds[b];
+      PyList_SetItem(m_bond_list, b, Py_BuildValue("iii",
+            bond.id1 - 1, bond.id2 - 1, bond.ref->order));
+    }
+
+    m_bonds.clear();
+  }
+};
+
+/*========================================================================*/
+
+/*
+ * Get all bonds within the given selection.
+ *
+ * Returns a list of (atm1, atm2, order) tuples, where atm1 and atm2 are
+ * the 0-based atom indices within the selection.
+ *
+ * Return value: New reference
+ */
+PyObject *MoleculeExporterGetPyBonds(PyMOLGlobals * G,
+    const char *selection, int state)
+{
+  SelectorTmp tmpsele1(G, selection);
+  int sele = tmpsele1.getIndex();
+  if (sele < 0)
+    return NULL;
+
+  int unblock = PAutoBlock(G);
+
+  MoleculeExporterPyBonds exporter;
+  exporter.init(G);
+  exporter.execute(sele, state);
+
+  if (PyErr_Occurred())
+    PyErr_Print();
+
+  PAutoUnblock(G, unblock);
+
+  return exporter.m_bond_list;
+}
+
+/*========================================================================*/
+
 /*
  * Creates a `chempy.models.Indexed` instance from a selection.
  *
@@ -1215,13 +1379,24 @@ unique_vla_ptr<char> MoleculeExporterGetStr(PyMOLGlobals * G,
  *
  */
 struct MoleculeExporterChemPy : public MoleculeExporter {
-  PyObject *m_model = NULL; // out
+  PyObject *m_model; // out
 
 protected:
-  int m_n_cs = 0; // number of coordinate sets
+  int m_n_cs; // number of coordinate sets
   float m_ref_tmp[3];
-  PyObject *m_atom_list = NULL;
+  PyObject *m_atom_list;
 
+public:
+  // quasi constructor
+  void init(PyMOLGlobals * G_) {
+    MoleculeExporter::init(G_);
+
+    m_model = NULL;
+    m_n_cs = 0;
+    m_atom_list = NULL;
+  }
+
+protected:
   int getMultiDefault() const {
     // single-entry format
     return cMolExportGlobal;
