@@ -35,12 +35,12 @@ Z* -------------------------------------------------------------------
 #include "PyMOL.h"
 #include "CGO.h"
 
-#define cPopUpLineHeight 17
-#define cPopUpTitleHeight 19
-#define cPopUpBarHeight 4
-#define cPopUpCharWidth 8
-#define cPopUpCharMargin 2
-#define cPopUpCharLift 2
+#define cPopUpLineHeight DIP2PIXEL(17)
+#define cPopUpTitleHeight DIP2PIXEL(19)
+#define cPopUpBarHeight DIP2PIXEL(4)
+#define cPopUpCharWidth DIP2PIXEL(8)
+#define cPopUpCharMargin DIP2PIXEL(2)
+#define cPopUpCharLift DIP2PIXEL(2)
 
 #define cChildDelay 0.25
 #define cPassiveDelay 0.45
@@ -88,19 +88,18 @@ static PyObject * SubGetItem(PyMOLGlobals * G, PyObject ** Sub, const int a) {
   if(!PyList_Check(elem)) {
     PBlock(G);
     elem = PyObject_CallObject(elem, NULL);
-    PUnblock(G);
 
-    ok_assert(2, elem);
+    if(PyErr_Occurred())
+      PyErr_Print();
 
     Py_DECREF(Sub[a]);
-    Py_INCREF(Sub[a] = elem);
+    Sub[a] = elem;
+
+    PUnblock(G);
   }
 
   return elem;
 
-ok_except2:
-  if(PyErr_Occurred())
-    PyErr_Print();
 ok_except1:
 #endif
   return NULL;
@@ -135,7 +134,7 @@ Block *PopUpNew(PyMOLGlobals * G, int x, int y, int last_x, int last_y,
   int mx, a, l, cl, cmx;
   int dim[2];
   PyObject *elem;
-  char *str, *c;
+  const char *str, *c;
   int blocked = PAutoBlock(G);
   int ui_light_bg = SettingGetGlobal_b(G, cSetting_internal_gui_mode);
   OOAlloc(G, CPopUp);
@@ -869,7 +868,7 @@ void PopUpDraw(Block * block ORTHOCGOARG)
 
           TextSetPos2i(G, xx, y + cPopUpCharLift);
           TextDrawChar(G, *(c++) ORTHOCGOARGVAR);
-          xx = xx + 8;
+          xx = xx + DIP2PIXEL(8);
         }
 
         if(I->Sub[a]) {
